from Bio import SeqIO


Get_COG_boxplot = '''
cd /Users/songweizhi/Desktop
python3 ~/PycharmProjects/BioSAK/BioSAK/boxplot_matrix_COG.py -in COG_func_stats -out COG_func_stats_df.txt -skip_1st_row -with_functional_description -in_percent
# important !!!
# please make sure the calculated proportions of genes in each COG category among detected HGTs are in the last row in COG_func_stats_df.txt, as values in the last row will be used to position the coloured triangles and squares in the plot.
Rscript ~/PycharmProjects/BioSAK/BioSAK/COG_boxplot_last1row.R -i COG_func_stats_df.txt -o COG_func_stats_df.png
# step_1
python3 boxplot_matrix_COG.py -in COG_func_stats -out COG_func_stats_df.txt -skip_1st_row -with_functional_description -in_percent
# step_2, important !!!
# please make sure sumarries for detected HGTs are in the last row in COG_func_stats_df.txt, as values in the last row will be used to position the coloured triangles and squares in the plot.
# step_3
Rscript COG_boxplot_last1row.R -i COG_func_stats_df.txt -o COG_func_stats_df.png
'''


def get_regions_to_ignore_from_barrnap_output(combined_barrnap_gff, ctg_len_dict):

    ctg_ignore_region_dict = {}

    for each_line in open(combined_barrnap_gff):
        if (not each_line.startswith('#')) and ('16S_rRNA' in each_line):
            each_line_split = each_line.strip().split('\t')
            ctg_id = each_line_split[0]
            ctg_len = ctg_len_dict[ctg_id]
            start_pos = int(each_line_split[3])
            end_pos = int(each_line_split[4])
            len_16s = end_pos - start_pos + 1
            left_gap = start_pos - 1
            right_gap = ctg_len - end_pos - 1

            if left_gap <= 50:
                if ctg_id not in ctg_ignore_region_dict:
                    ctg_ignore_region_dict[ctg_id] = {'left_end'}
                else:
                    ctg_ignore_region_dict[ctg_id].add('left_end')

            if right_gap <= 50:
                if ctg_id not in ctg_ignore_region_dict:
                    ctg_ignore_region_dict[ctg_id] = {'right_end'}
                else:
                    ctg_ignore_region_dict[ctg_id].add('right_end')

    return ctg_ignore_region_dict


def get_unlinked_mag_end_seq(ref_in, ref_in_end_seq, end_seq_len, ctg_ignore_region_dict_rd1):
    ctg_ignore_region_dict_rd2 = dict()

    # get ref seqs subset
    ref_subset_handle = open(ref_in_end_seq, 'w')
    for ref_seq in SeqIO.parse(ref_in, 'fasta'):

        ref_seq_id = ref_seq.id
        ref_seq_len = len(ref_seq.seq)

        if ref_seq_len < end_seq_len * 2:
            ref_subset_handle.write('>%s\n' % ref_seq_id)
            ref_subset_handle.write('%s\n' % ref_seq.seq)

            # add to ctg_ignore_region_dict_rd2
            if ref_seq_id in ctg_ignore_region_dict_rd1:
                ctg_ignore_region_dict_rd2[ref_seq_id] = ctg_ignore_region_dict_rd1[ref_seq_id]
        else:
            ref_seq_left_end_id = '%s_l' % ref_seq_id
            ref_seq_right_end_id = '%s_r' % ref_seq_id
            ref_seq_left_end = ref_seq.seq[:end_seq_len]
            ref_seq_right_end = ref_seq.seq[-end_seq_len:]

            # write out left end
            ref_subset_handle.write('>%s\n' % ref_seq_left_end_id)
            ref_subset_handle.write('%s\n' % ref_seq_left_end)

            # write out right end
            ref_subset_handle.write('>%s\n' % ref_seq_right_end_id)
            ref_subset_handle.write('%s\n' % ref_seq_right_end)

            # add to ctg_ignore_region_dict_rd2
            if ref_seq_id in ctg_ignore_region_dict_rd1:
                current_seq_to_ignore_ends = ctg_ignore_region_dict_rd1[ref_seq_id]
                for end_to_ignore in current_seq_to_ignore_ends:
                    if end_to_ignore == 'left_end':
                        ctg_ignore_region_dict_rd2[ref_seq_left_end_id] = {'left_end'}
                    if end_to_ignore == 'right_end':
                        ctg_ignore_region_dict_rd2[ref_seq_right_end_id] = {'right_end'}
    ref_subset_handle.close()

    return ctg_ignore_region_dict_rd2


step_2_wd                                   = '/Users/songweizhi/Desktop/tunning_rd2'
combined_input_gnms                         = '%s/file_in/3_Oral_refined_MAGs_combined.fa'                  % step_2_wd
combined_input_gnms_no_ending_16s           = '%s/file_in/3_Oral_refined_MAGs_combined_no_ending_16s.fa'    % step_2_wd
end_seq_len = 500
combined_1st_round_unlinked_mags            = '%s/file_in/round_1_unlinked_gnm.fa'                          % step_2_wd
combined_1st_round_unlinked_mag_end_seq     = '%s/file_in/round_1_unlinked_gnm_end_%sbp.fa'                 % (step_2_wd, end_seq_len)
combined_barrnap_gff                        = '%s/file_in/combined_barrnap.gff'                             % step_2_wd


ctg_len_dict = {}
for each_ctg_record in SeqIO.parse(combined_input_gnms, 'fasta'):
    ctg_len_dict[each_ctg_record.id] = len(each_ctg_record.seq)

ctg_ended_16s_to_keep_part_dict = dict()
for each_line in open(combined_barrnap_gff):
    if (not each_line.startswith('#')) and ('16S_rRNA' in each_line):
        each_line_split = each_line.strip().split('\t')
        ctg_id = each_line_split[0]
        ctg_len = ctg_len_dict[ctg_id]
        start_pos = int(each_line_split[3])
        end_pos = int(each_line_split[4])
        len_16s = end_pos - start_pos + 1
        left_gap = start_pos - 1
        right_gap = ctg_len - end_pos - 1
        if left_gap <= 100:
            ctg_ended_16s_to_keep_part_dict[ctg_id] = '%s-%s' % ((end_pos + 1), ctg_len)
        elif right_gap <= 100:
            ctg_ended_16s_to_keep_part_dict[ctg_id] = '%s-%s' % (1, start_pos)

combined_input_gnms_no_ending_16s_handle = open(combined_input_gnms_no_ending_16s, 'w')
for each_ctg in SeqIO.parse(combined_input_gnms, 'fasta'):
    ctg_id = each_ctg.id
    ctg_seq = str(each_ctg.seq)

    if ctg_id not in ctg_ended_16s_to_keep_part_dict:
        combined_input_gnms_no_ending_16s_handle.write('>%s\n' % ctg_id)
        combined_input_gnms_no_ending_16s_handle.write('%s\n' % ctg_seq)
    else:
        keep_region_str = ctg_ended_16s_to_keep_part_dict[ctg_id].split('-')
        keep_region_l = int(keep_region_str[0])
        keep_region_r = int(keep_region_str[1])
        combined_input_gnms_no_ending_16s_handle.write('>%s\n' % ctg_id)
        combined_input_gnms_no_ending_16s_handle.write('%s\n' % ctg_seq[(keep_region_l - 1): keep_region_r])

combined_input_gnms_no_ending_16s_handle.close()

# ctg_ignore_region_dict = get_regions_to_ignore_from_barrnap_output(combined_barrnap_gff, ctg_len_dict)
# print(ctg_ignore_region_dict)
#
# ctg_ignore_region_dict_rd2 = get_unlinked_mag_end_seq(combined_1st_round_unlinked_mags,
#                                                       combined_1st_round_unlinked_mag_end_seq, end_seq_len,
#                                                       ctg_ignore_region_dict)




pwd_samfile = '/Users/songweizhi/Desktop/input_reads_to_16S.sam'
pwd_samfile = 'round_1_unlinked_gnm.sam'


id_to_seq_dict = {}
for each_read in open(pwd_samfile):
    if not each_read.startswith('@'):
        each_read_split = each_read.strip().split('\t')
        read_id = each_read_split[0]
        read_seq = each_read_split[9]
        if read_id not in id_to_seq_dict:
            id_to_seq_dict[read_id] = {read_seq}
        else:
            id_to_seq_dict[read_id].add(read_seq)

for each_read in id_to_seq_dict:
    if len(id_to_seq_dict[each_read]) > 1:
        for each_seq in id_to_seq_dict[each_read]:
            print('%s\t%s' % (each_read, each_seq))
        print()



import os

full_name_file = '/Users/songweizhi/Desktop/full_name_sorted.txt'
Kelp_sample_id = '/Users/songweizhi/Desktop/Kelp_sample_id.txt'
full_name_file = 'full_name_sorted.txt'
Kelp_sample_id = 'Kelp_sample_id.txt'


sample_file_dict = {}
for file in open(full_name_file):
    file = file.strip()
    sample_id = file.split('_')[0]
    if sample_id not in sample_file_dict:
        if 'pairedForward' in file:
            sample_file_dict[sample_id] = [[file],[]]
        if 'pairedReverse' in file:
            sample_file_dict[sample_id] = [[],[file]]
    else:
        if 'pairedForward' in file:
            sample_file_dict[sample_id][0].append(file)
        if 'pairedReverse' in file:
            sample_file_dict[sample_id][1].append(file)

for sample in sample_file_dict:
    current_sample_r1 = sorted(sample_file_dict[sample][0])
    current_sample_r2 = sorted(sample_file_dict[sample][1])
    current_sample_r1_no_gz = [i[:-3] for i in current_sample_r1]
    current_sample_r2_no_gz = [i[:-3] for i in current_sample_r2]

    for r1_gz in current_sample_r1:
        os.system('gunzip %s' % r1_gz)
    for r2_gz in current_sample_r2:
        os.system('gunzip %s' % r2_gz)

    os.system('cat %s > %s_R1.fastq' % (' '.join(current_sample_r1_no_gz), sample))
    os.system('cat %s > %s_R2.fastq' % (' '.join(current_sample_r2_no_gz), sample))

    for r1_gz in current_sample_r1_no_gz:
        os.system('rm %s' % r1_gz)
    for r2_gz in current_sample_r2_no_gz:
        os.system('rm %s' % r2_gz)



co_assembly_dict = {}
for co_assembly in open(Kelp_sample_id):
    co_assembly_split = co_assembly.strip().split('\t')
    if co_assembly_split[0] not in co_assembly_dict:
        co_assembly_dict[co_assembly_split[0]] = [co_assembly_split[1]]
    else:
        co_assembly_dict[co_assembly_split[0]].append(co_assembly_split[1])

for each_co_assembly in co_assembly_dict:
    if len(co_assembly_dict[each_co_assembly]) == 1:
        os.system('mv %s_R1.fastq %s_R1.fastq' % (co_assembly_dict[each_co_assembly][0], each_co_assembly))
        os.system('mv %s_R2.fastq %s_R2.fastq ' % (co_assembly_dict[each_co_assembly][0], each_co_assembly))
    else:
        os.system('cat %s > %s_R1.fastq' % (' '.join([(i + '_R1.fastq') for i in co_assembly_dict[each_co_assembly]]), each_co_assembly))
        os.system('cat %s > %s_R2.fastq' % (' '.join([(i + '_R2.fastq') for i in co_assembly_dict[each_co_assembly]]), each_co_assembly))
    for each_id in co_assembly_dict[each_co_assembly]:
        os.system('rm %s_R1.fastq' % each_id)
        os.system('rm %s_R2.fastq' % each_id)

    os.system('fq2fa %s_R1.fastq %s_R1.fasta' % (each_co_assembly, each_co_assembly))
    os.system('fq2fa %s_R2.fastq %s_R2.fasta' % (each_co_assembly, each_co_assembly))

    os.system('rm %s_R1.fastq' % each_co_assembly)
    os.system('rm %s_R2.fastq' % each_co_assembly)







name_list = ['a', 'b', 'c', 'd']
value_list = [4, 7, 2, 0]

name_list = ['a']
value_list = [4]

sorted_best_16s_list = [[seq_id, mean_iden] for mean_iden, seq_id in sorted(zip(value_list, name_list), reverse=True)]

best_matched_marker = sorted_best_16s_list[0][0]
best_matched_marker_iden = sorted_best_16s_list[0][1]


print(best_matched_marker)
print(best_matched_marker_iden)



import os


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def overlap_between_list(list_1, list_2):

    overlap = False
    for list_1_element in list_1:
        if list_1_element in list_2:
            overlap = True

    return overlap


def gnm_level_asessment(linked_mag_dict_rd1):

    rd1_correct_num = 0
    rd1_wrong_gnm = set()
    rd1_unknown_gnm = set()
    rd1_ambiguous_gnm = set()
    for each_rd_1_mag in linked_mag_dict_rd1:
        mag_assess = linked_mag_dict_rd1[each_rd_1_mag]
        if len(mag_assess) == 1:
            if mag_assess == {'Correct'}:
                rd1_correct_num += 1
            elif mag_assess == {'Wrong'}:
                rd1_wrong_gnm.add(each_rd_1_mag)
            elif mag_assess == {'Unknown'}:
                rd1_unknown_gnm.add(each_rd_1_mag)
        elif len(mag_assess) == 2:
            if ('Correct' in mag_assess) and ('Unknown' in mag_assess):
                rd1_correct_num += 1
            if ('Correct' in mag_assess) and ('Wrong' in mag_assess):
                rd1_ambiguous_gnm.add(each_rd_1_mag)
            if ('Wrong' in mag_assess) and ('Unknown' in mag_assess):
                rd1_wrong_gnm.add(each_rd_1_mag)
        elif len(mag_assess) == 3:
            rd1_ambiguous_gnm.add(each_rd_1_mag)

    return rd1_correct_num, rd1_unknown_gnm, rd1_wrong_gnm, rd1_ambiguous_gnm


########################################################################################################################

wd = '/Users/songweizhi/Desktop/assess_linkages_GI'

########## reference to cluster ##########

drep_ani_cutoff             = 97
drep_cdb_file               = '%s/file_in/Cdb_%s.csv'                                       % (wd, drep_ani_cutoff)
ref_to_strain_file          = '%s/file_in/ref_to_strain.txt'                                % wd

########## bin to reference ##########

parse_blastn_bin_vs_ref     = False  # True or False
blastn_bin_vs_ref           = '%s/file_in/bin_vs_ref.tab'                                   % wd
iden_cutoff                 = 99.5
aln_len_cutoff              = 1500
cov_q_cutoff                = 90
min_match_length            = 524288  # 100 Kbp  102400
mag_metadata                = '%s/file_in/MAG_metadata.txt'                                 % wd
pwd_plot_sankey_R           = '%s/file_in/get_sankey_plot.R' % wd

########## 16S to reference ##########

perform_blastn_16s_vs_refs  = False  # True or False
combined_GI_ref_16S         = '%s/file_in/combined_GI_ref_16S.ffn'           % wd
matam_16s_seqs              = '%s/file_in/GI_138_16S_0.999.fasta'            % wd
matam_16s_blastn            = '%s/file_in/GI_138_16S_0.999_vs_ref.tab'       % wd
iden_cutoff_16s             = 99.5  # 99.3 (best), 99.5
aln_len_cutoff_16s          = 500
cov_q_cutoff_16s            = 70
'''


3_GI_138_subsample_75_842	OTU_97.8962.0_16S_4	99.802	1512	3	0	5	1516	1	1512	0.0	2713	1519	1512
3_GI_138_subsample_75_847	OTU_97.8962.0_16S_1	99.669	1512	5	0	5	1516	1	1512	0.0	2704	1519	1512

'''
########## assessment results ##########

MarkerMAG_linkages          = '%s/GI_0715_138_60_60_linkages_by_genome.txt'             % wd
linkages_from_rd1           = False

'''


GI_0524_138_45_45_stats_combined_filtered.txt	Rd_1	|	118	101	10	7	101/108(93.52)	|	36	30	1	3	2	30/35(85.71)
GI_0524_138_55_55_stats_combined_filtered.txt	Rd_1	|	115	98	11	6	98/104(94.23)	|	35	30	1	3	1	30/34(88.24)
GI_0524_138_60_60_stats_combined_filtered.txt	Rd_1	|	111	102	7	2	102/104(98.08)	|	29	27	0	2	0	27/29(93.1)

'''
############################################### define file/folder name ################################################

# bin to reference
bin_vs_ref_txt              = '%s/file_in/bin_vs_ref_imag%s.txt'                    % (wd, iden_cutoff)
stats_bin_to_ref_txt        = '%s/file_in/stats_bin_to_ref_imag%s.txt'              % (wd, iden_cutoff)
stats_ref_to_bin_txt        = '%s/file_in/stats_ref_to_bin_imag%s.txt'              % (wd, iden_cutoff)
stats_bin_to_cluster_txt    = '%s/file_in/stats_bin_to_cluster_ani%s_imag%s.txt'    % (wd, drep_ani_cutoff, iden_cutoff)
stats_cluster_to_bin_txt    = '%s/file_in/stats_cluster_to_bin_ani%s_imag%s.txt'    % (wd, drep_ani_cutoff, iden_cutoff)

# assessment results
linkage_file_path, linkage_file_basename, linkage_file_extension = sep_path_basename_ext(MarkerMAG_linkages)
MarkerMAG_linkages_assessed = '%s/%s_with_assessment_ani%s_imag%s_i16S%s.txt'   % (linkage_file_path, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)
wrong_linkages_txt          = '%s/%s_wrong_ani%s_imag%s_i16S%s.txt'             % (linkage_file_path, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)
unknown_linkages_txt        = '%s/%s_unknown_ani%s_imag%s_i16S%s.txt'           % (linkage_file_path, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)


################################################# reference to cluster #################################################

cluster_to_ref_dict = {}
ref_to_cluster_dict = {}
for each_ref in open(drep_cdb_file):
    if not each_ref.startswith('genome,secondary_cluster'):
        each_ref_split = each_ref.strip().split(',')
        ref_file_name = each_ref_split[0]
        ref_file_name_no_ext = '.'.join(ref_file_name.split('.')[:-1])
        ref_cluster = 'C' + each_ref_split[1]
        ref_to_cluster_dict[ref_file_name_no_ext] = ref_cluster
        if ref_cluster not in cluster_to_ref_dict:
            cluster_to_ref_dict[ref_cluster] = [ref_file_name_no_ext]
        else:
            cluster_to_ref_dict[ref_cluster].append(ref_file_name_no_ext)


################################################### bin to reference ###################################################

bin_ref_connector = '__|__'

# get ref_to_strain_dict
ref_to_strain_dict = {}
for ref in open(ref_to_strain_file):
    ref_split = ref.strip().split('\t')
    ref_to_strain_dict[ref_split[0]] = ref_split[1]

if parse_blastn_bin_vs_ref is True:
    bin_vs_ref_dict = {}
    for match in open(blastn_bin_vs_ref):
        match_split = match.strip().split('\t')
        query = match_split[0]
        query_genome = '_'.join(query.split('_')[:2])
        subject = match_split[1]
        subject_genome = '_'.join(subject.split('_')[:2])
        iden = float(match_split[2])
        aln_len = int(match_split[3])
        query_len = int(match_split[12])
        subject_len = int(match_split[13])
        coverage_q = float(aln_len) * 100 / float(query_len)
        if (iden >= iden_cutoff) and (aln_len > aln_len_cutoff) and (coverage_q >= cov_q_cutoff):
            key_bin_ref = '%s%s%s' % (query_genome, bin_ref_connector, subject_genome)
            if key_bin_ref not in bin_vs_ref_dict:
                bin_vs_ref_dict[key_bin_ref] = aln_len
            else:
                bin_vs_ref_dict[key_bin_ref] += aln_len

    bin_vs_ref_dict_filtered = {}
    for each_key in bin_vs_ref_dict:
        if bin_vs_ref_dict[each_key] >= min_match_length:
            bin_vs_ref_dict_filtered[each_key] = bin_vs_ref_dict[each_key]

    # write out linkages
    linkage_txt_handle = open(bin_vs_ref_txt, 'w')
    linkage_txt_handle.write('Bin,Ref,Length\n')
    bin_to_ref_dict = {}
    ref_to_bin_dict = {}
    bin_to_cluster_dict = {}
    cluster_to_bin_dict = {}
    for each_linkage in bin_vs_ref_dict_filtered:
        linkage_split = each_linkage.split(bin_ref_connector)
        bin_id = linkage_split[0]
        ref_id = linkage_split[1]
        ref_cluster = ref_to_cluster_dict[ref_id]

        if bin_id not in bin_to_ref_dict:
            bin_to_ref_dict[bin_id] = {ref_id}
        else:
            bin_to_ref_dict[bin_id].add(ref_id)

        if bin_id not in bin_to_cluster_dict:
            bin_to_cluster_dict[bin_id] = {ref_cluster}
        else:
            bin_to_cluster_dict[bin_id].add(ref_cluster)

        if ref_id not in ref_to_bin_dict:
            ref_to_bin_dict[ref_id] = {bin_id}
        else:
            ref_to_bin_dict[ref_id].add(bin_id)

        if ref_cluster not in cluster_to_bin_dict:
            cluster_to_bin_dict[ref_cluster] = {bin_id}
        else:
            cluster_to_bin_dict[ref_cluster].add(bin_id)

        linkage_txt_handle.write('%s,%s,%s\n' % (bin_id, ref_id, bin_vs_ref_dict_filtered[each_linkage]))
    linkage_txt_handle.close()

    # visualize
    cmd_sankey_bin_to_ref = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, bin_vs_ref_txt, 600, 1800)
    os.system(cmd_sankey_bin_to_ref)

    bin_to_ref_txt_handle = open(stats_bin_to_ref_txt, 'w')
    for each_bin in bin_to_ref_dict:
        bin_to_ref_txt_handle.write('%s\t%s\n' % (each_bin, ','.join(bin_to_ref_dict[each_bin])))
    bin_to_ref_txt_handle.close()

    ref_to_bin_txt_handle = open(stats_ref_to_bin_txt, 'w')
    for each_ref in ref_to_bin_dict:
        ref_to_bin_txt_handle.write('%s\t%s\t%s\n' % (each_ref, ','.join(ref_to_bin_dict[each_ref]), ref_to_strain_dict[each_ref]))
    ref_to_bin_txt_handle.close()

    stats_bin_to_cluster_txt_handle = open(stats_bin_to_cluster_txt, 'w')
    for each_bin in bin_to_cluster_dict:
        stats_bin_to_cluster_txt_handle.write('%s\t%s\n' % (each_bin, ','.join(bin_to_cluster_dict[each_bin])))
    stats_bin_to_cluster_txt_handle.close()

    stats_cluster_to_bin_txt_handle = open(stats_cluster_to_bin_txt, 'w')
    for each_ref in cluster_to_bin_dict:
        stats_cluster_to_bin_txt_handle.write('%s\t%s\n' % (each_ref, ','.join(cluster_to_bin_dict[each_ref])))
    stats_cluster_to_bin_txt_handle.close()

bin_to_cluster_dict = {}
for each_match in open(stats_bin_to_cluster_txt):
    each_match_split = each_match.strip().split('\t')
    bin_to_cluster_dict[each_match_split[0]] = {i for i in each_match_split[1].split(',')}

cluster_to_bin_dict = {}
for each_bin in bin_to_cluster_dict:
    matched_clusters =  bin_to_cluster_dict[each_bin]
    for matched_cluster in matched_clusters:
        if matched_cluster not in cluster_to_bin_dict:
            cluster_to_bin_dict[matched_cluster] = {each_bin}
        else:
            cluster_to_bin_dict[matched_cluster].add(each_bin)


################################################### 16S to reference ###################################################

blast_parameters = '-evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 1'
blast_cmd = 'blastn -query %s -subject %s -out %s %s' % (matam_16s_seqs, combined_GI_ref_16S, matam_16s_blastn, blast_parameters)
if perform_blastn_16s_vs_refs is True:
    os.system(blast_cmd)

# get matam_16s_to_cluster_dict
matam_16s_to_cluster_dict = {}
for match in open(matam_16s_blastn):
    match_split = match.strip().split('\t')
    query = match_split[0]
    subject = match_split[1]
    subject_gnm = subject.split('_16S_')[0]
    subject_cluster = ref_to_cluster_dict[subject_gnm]
    iden = float(match_split[2])
    aln_len = int(match_split[3])
    query_len = int(match_split[12])
    subject_len = int(match_split[13])
    coverage_q = float(aln_len) * 100 / float(query_len)
    if (iden >= iden_cutoff_16s) and (aln_len > aln_len_cutoff_16s) and (coverage_q >= cov_q_cutoff_16s):
        if query not in matam_16s_to_cluster_dict:
            matam_16s_to_cluster_dict[query] = {subject_cluster}
        else:
            matam_16s_to_cluster_dict[query].add(subject_cluster)

cluster_to_matam_16s_dict = {}
for matam_16s in matam_16s_to_cluster_dict:
    matched_clusters =  matam_16s_to_cluster_dict[matam_16s]
    for matched_cluster in matched_clusters:
        if matched_cluster not in cluster_to_matam_16s_dict:
            cluster_to_matam_16s_dict[matched_cluster] = {matam_16s}
        else:
            cluster_to_matam_16s_dict[matched_cluster].add(matam_16s)

###################################################### assessment ######################################################

MarkerMAG_linkages_assessed_handle = open(MarkerMAG_linkages_assessed, 'w')
wrong_linkages_txt_handle = open(wrong_linkages_txt, 'w')
unknown_linkages_txt_handle = open(unknown_linkages_txt, 'w')
linkage_num_right = 0
linkage_num_wrong = 0
linkage_num_unknown = 0
linkage_assessment_dict = {}
for each_linkage in open(MarkerMAG_linkages):
    if ('MarkerGene\tGenomicSeq\tLinkage\tStep' in each_linkage) or ('MarkerGene,GenomicSeq,Number' in each_linkage):
        MarkerMAG_linkages_assessed_handle.write('MarkerGene\tGenomicSeq\tLinkage\tStep\tAssessment\n')
    else:
        id_16s = ''
        id_mag = ''
        link_num = ''
        if linkages_from_rd1 is False:
            each_linkage_split = each_linkage.strip().split('\t')
            id_16s = each_linkage_split[0]
            id_mag = each_linkage_split[1]
            link_num = each_linkage_split[2]
        else:
            each_linkage_split = each_linkage.strip().split(',')
            id_16s = each_linkage_split[0][12:]
            id_mag = each_linkage_split[1][12:]
            link_num = each_linkage_split[2]

        key_16s_mag = '%s___%s' % (id_16s, id_mag)

        matched_cluster_16s = {}
        if id_16s in matam_16s_to_cluster_dict:
            matched_cluster_16s = matam_16s_to_cluster_dict[id_16s]

        matched_cluster_mag = {}
        if id_mag in bin_to_cluster_dict:
            matched_cluster_mag = bin_to_cluster_dict[id_mag]

        if (matched_cluster_16s != {}) and (matched_cluster_mag != {}):

            if overlap_between_list(matched_cluster_mag, matched_cluster_16s) is True:
                linkage_num_right += 1

                if linkages_from_rd1 is False:
                    MarkerMAG_linkages_assessed_handle.write('%s\tCorrect\n' % each_linkage.strip())
                else:
                    MarkerMAG_linkages_assessed_handle.write('%s\t%s\t%s\tS1\tCorrect\n' % (id_16s, id_mag, link_num))

                if id_mag not in linkage_assessment_dict:
                    linkage_assessment_dict[id_mag] = ['Correct']
                else:
                    linkage_assessment_dict[id_mag].append('Correct')

            else:
                linkage_num_wrong += 1

                if linkages_from_rd1 is False:
                    MarkerMAG_linkages_assessed_handle.write('%s\tWrong\n' % each_linkage.strip())
                else:
                    MarkerMAG_linkages_assessed_handle.write('%s\t%s\t%s\tS1\tWrong\n' % (id_16s, id_mag, link_num))

                if id_mag not in linkage_assessment_dict:
                    linkage_assessment_dict[id_mag] = ['Wrong']
                else:
                    linkage_assessment_dict[id_mag].append('Wrong')

                for each_16s_cluster in matched_cluster_16s:
                    current_16s_cluster_ref = cluster_to_ref_dict[each_16s_cluster]
                    for each_ref_1 in current_16s_cluster_ref:
                        wrong_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_wrong, id_16s, id_mag, '16S', each_16s_cluster, each_ref_1, ref_to_strain_dict[each_ref_1]))

                for each_mag_cluster in matched_cluster_mag:
                    current_mag_cluster_ref = cluster_to_ref_dict[each_mag_cluster]
                    for each_ref_2 in current_mag_cluster_ref:
                        wrong_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_wrong, id_16s, id_mag, 'MAG', each_mag_cluster, each_ref_2, ref_to_strain_dict[each_ref_2]))
                wrong_linkages_txt_handle.write('\n')
        else:
            linkage_num_unknown += 1

            if linkages_from_rd1 is False:
                MarkerMAG_linkages_assessed_handle.write('%s\tUnknown\n' % each_linkage.strip())
            else:
                MarkerMAG_linkages_assessed_handle.write('%s\t%s\t%s\tS1\tUnknown\n' % (id_16s, id_mag, link_num))

            if id_mag not in linkage_assessment_dict:
                linkage_assessment_dict[id_mag] = ['Unknown']
            else:
                linkage_assessment_dict[id_mag].append('Unknown')

            for each_16s_cluster in matched_cluster_16s:
                current_16s_cluster_ref = cluster_to_ref_dict[each_16s_cluster]
                for each_ref_1 in current_16s_cluster_ref:
                    unknown_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_unknown, id_16s, id_mag, '16S', each_16s_cluster, each_ref_1, ref_to_strain_dict[each_ref_1]))
            for each_mag_cluster in matched_cluster_mag:
                current_mag_cluster_ref = cluster_to_ref_dict[each_mag_cluster]
                for each_ref_2 in current_mag_cluster_ref:
                    unknown_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_unknown, id_16s, id_mag, 'MAG', each_mag_cluster, each_ref_2, ref_to_strain_dict[each_ref_2]))
            unknown_linkages_txt_handle.write('\n')
wrong_linkages_txt_handle.close()
unknown_linkages_txt_handle.close()
MarkerMAG_linkages_assessed_handle.close()


# get assess stats at mag level
linked_mag_dict_rd1 = {}
linked_mag_dict_rd2 = {}
total_linkage_num_rd1 = 0
total_linkage_num_rd2 = 0
correct_link_rd1 = 0
correct_link_rd2 = 0
wrong_link_rd1 = 0
wrong_link_rd2 = 0
unknown_link_rd1 = 0
unknown_link_rd2 = 0
for each_link in open(MarkerMAG_linkages_assessed):
    if not each_link.startswith('MarkerGene	GenomicSeq	Linkage	Step	Assessment'):
        each_link_split = each_link.strip().split('\t')
        gnm_id = each_link_split[1]
        linked_rd = each_link_split[3]
        assessment = each_link_split[4]
        if linked_rd == 'S1':
            total_linkage_num_rd1 += 1

            if assessment == 'Correct':
                correct_link_rd1 += 1
            if assessment == 'Wrong':
                wrong_link_rd1 += 1
            if assessment == 'Unknown':
                unknown_link_rd1 += 1

            if gnm_id not in linked_mag_dict_rd1:
                linked_mag_dict_rd1[gnm_id] = {assessment}
            else:
                linked_mag_dict_rd1[gnm_id].add(assessment)
        if linked_rd == 'S2':
            total_linkage_num_rd2 += 1

            if assessment == 'Correct':
                correct_link_rd2 += 1
            if assessment == 'Wrong':
                wrong_link_rd2 += 1
            if assessment == 'Unknown':
                unknown_link_rd2 += 1

            if gnm_id not in linked_mag_dict_rd1:
                linked_mag_dict_rd2[gnm_id] = {assessment}
            else:
                linked_mag_dict_rd2[gnm_id].add(assessment)


total_linkage_num_both = total_linkage_num_rd1 + total_linkage_num_rd2
total_linked_mag_num = len(linked_mag_dict_rd1) + len(linked_mag_dict_rd2)
rd1_correct_num, rd1_unknown_gnm, rd1_wrong_gnm, rd1_ambiguous_gnm = gnm_level_asessment(linked_mag_dict_rd1)
rd2_correct_num, rd2_unknown_gnm, rd2_wrong_gnm, rd2_ambiguous_gnm = gnm_level_asessment(linked_mag_dict_rd2)
correct_num_both        = rd1_correct_num + rd2_correct_num
unknown_gnm_both_num    = len(rd1_unknown_gnm) + len(rd2_unknown_gnm)
wrong_gnm_both_num      = len(rd1_wrong_gnm) + len(rd2_wrong_gnm)
ambiguous_gnm_both_num  = len(rd1_ambiguous_gnm) + len(rd2_ambiguous_gnm)

accuracy_str_rd1   = '%s/%s(%s)' % (rd1_correct_num,  (len(linked_mag_dict_rd1) - len(rd1_unknown_gnm)), float("{0:.2f}".format(rd1_correct_num*100/(len(linked_mag_dict_rd1) - len(rd1_unknown_gnm)))))
accuracy_str_rd2 = '0/0(0)'
if (len(linked_mag_dict_rd2) - len(rd2_unknown_gnm)) > 0:
    accuracy_str_rd2   = '%s/%s(%s)' % (rd2_correct_num,  (len(linked_mag_dict_rd2) - len(rd2_unknown_gnm)), float("{0:.2f}".format(rd2_correct_num*100/(len(linked_mag_dict_rd2) - len(rd2_unknown_gnm)))))
accuracy_str_both  = '%s/%s(%s)' % (correct_num_both, (total_linked_mag_num - unknown_gnm_both_num), float("{0:.2f}".format(correct_num_both*100/(total_linked_mag_num - unknown_gnm_both_num))))

correct_link_both = correct_link_rd1 + correct_link_rd2
wrong_link_both   = wrong_link_rd1 + wrong_link_rd2
unknown_link_both = unknown_link_rd1 + unknown_link_rd2

accuracy_str_rd1_link_level   = '%s/%s(%s)' % (correct_link_rd1,  (total_linkage_num_rd1 - unknown_link_rd1), float("{0:.2f}".format(correct_link_rd1*100/(total_linkage_num_rd1 - unknown_link_rd1))))
accuracy_str_rd2_link_level = '0/0(0)'
if (total_linkage_num_rd2 - unknown_link_rd2) > 0:
    accuracy_str_rd2_link_level   = '%s/%s(%s)' % (correct_link_rd2,  (total_linkage_num_rd2 - unknown_link_rd2), float("{0:.2f}".format(correct_link_rd2*100/(total_linkage_num_rd2 - unknown_link_rd2))))
accuracy_str_both_link_level  = '%s/%s(%s)' % (correct_link_both, (total_linkage_num_both - unknown_link_both), float("{0:.2f}".format(correct_link_both*100/(total_linkage_num_both - unknown_link_both))))

prefix = os.path.basename(MarkerMAG_linkages).split('_identified_linkages_genome_level')[0]
print('%s\tRound\t|\tLink\tYes\tNA\tNo\tAccuracy\t|\tMAG\tYes\tNA\tNo\tY/N\tAccuracy' % prefix)
print('%s\tRd_1\t|\t%s\t%s\t%s\t%s\t%s\t|\t%s\t%s\t%s\t%s\t%s\t%s' % (prefix, total_linkage_num_rd1,  correct_link_rd1,  unknown_link_rd1,  wrong_link_rd1,  accuracy_str_rd1_link_level,  len(linked_mag_dict_rd1), rd1_correct_num,  len(rd1_unknown_gnm), len(rd1_wrong_gnm), len(rd1_ambiguous_gnm), accuracy_str_rd1))
print('%s\tRd_2\t|\t%s\t%s\t%s\t%s\t%s\t|\t%s\t%s\t%s\t%s\t%s\t%s' % (prefix, total_linkage_num_rd2,  correct_link_rd2,  unknown_link_rd2,  wrong_link_rd2,  accuracy_str_rd2_link_level,  len(linked_mag_dict_rd2), rd2_correct_num,  len(rd2_unknown_gnm), len(rd2_wrong_gnm), len(rd2_ambiguous_gnm), accuracy_str_rd2))
print('%s\tBoth\t|\t%s\t%s\t%s\t%s\t%s\t|\t%s\t%s\t%s\t%s\t%s\t%s' % (prefix, total_linkage_num_both, correct_link_both, unknown_link_both, wrong_link_both, accuracy_str_both_link_level, total_linked_mag_num,      correct_num_both, unknown_gnm_both_num, wrong_gnm_both_num, ambiguous_gnm_both_num, accuracy_str_both))

########################################################################################################################

print(cluster_to_matam_16s_dict['C109_0'])
print(cluster_to_bin_dict['C110_0'])



# print(matam_16s_to_cluster_dict['3_GI_138_subsample_75_842'])
# print(matam_16s_to_cluster_dict['3_GI_138_subsample_75_847'])
# print(bin_to_cluster_dict['Refined_52'])
# print(cluster_to_bin_dict['C47_1'])
# print(cluster_to_bin_dict['C48_0'])




def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if each_element.isalpha() is True:
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


unmapped_paired_reads_blastn    = '/Users/songweizhi/Desktop/unmapped_paired_reads_blast.txt'
reads_iden_cutoff               = 100  # s1_ri
reads_cov_cutoff                = 90   # s1_rc


pwd_samfile = '/Users/songweizhi/Desktop/BH_ER_050417_all_depth_assemblies.dereplicated.sam'
perfect_match_min_cigar_M_pct       = 90
perfect_match_max_cigar_S_pct       = 10
clipping_reads_not_matched_part_seq = '/Users/songweizhi/Desktop/test.fa'
error_rate_cutoff = 0.005


# export clipping mapped reads and perfectly mapped reads
perfect_num = 0
all_mapped_reads_set = set()
clipping_mapped_reads_list = set()
clipping_reads_mapped_part_dict = {}
perfectly_mapped_reads_dict = {}
clipping_reads_not_matched_part_seq_handle = open(clipping_reads_not_matched_part_seq, 'w')
for each_read in open(pwd_samfile):

    qualified_perfect_match = False

    if not each_read.startswith('@'):
        each_read_split = each_read.strip().split('\t')
        read_id = each_read_split[0]
        read_id_base = '.'.join(read_id.split('.')[:-1])
        read_strand = read_id.split('.')[-1]
        ref_id = each_read_split[2]
        ref_id_with_prefix = 'MarkerGene__%s' % each_read_split[2]
        ref_pos = int(each_read_split[3])
        cigar = each_read_split[5]
        read_seq = each_read_split[9]
        cigar_splitted = cigar_splitter(cigar)
        read_id_with_ref_pos = '%s__x__%s__x__%s' % (read_id, ref_id, ref_pos)

        all_mapped_reads_set.add(read_id)

        # for perfectly mapped reads
        if ('M' in cigar) and (len(cigar_splitted) == 1):
            if read_id_base not in perfectly_mapped_reads_dict:
                perfectly_mapped_reads_dict[read_id_base] = {read_strand: [ref_id_with_prefix]}
            else:
                if read_strand not in perfectly_mapped_reads_dict[read_id_base]:
                    perfectly_mapped_reads_dict[read_id_base][read_strand] = [ref_id_with_prefix]
                else:
                    perfectly_mapped_reads_dict[read_id_base][read_strand].append(ref_id_with_prefix)

            perfect_num += 1

        # for clipping mapped reads
        if ('S' in cigar) and (len(cigar_splitted) == 2):
            cigar_M_len = 0
            cigar_S_len = 0
            if cigar_splitted[0][-1] == 'M':
                cigar_M_len = int(cigar_splitted[0][:-1])
                cigar_S_len = int(cigar_splitted[1][:-1])
            if cigar_splitted[1][-1] == 'M':
                cigar_M_len = int(cigar_splitted[1][:-1])
                cigar_S_len = int(cigar_splitted[0][:-1])

            cigar_M_pct = cigar_M_len * 100 / (cigar_M_len + cigar_S_len)
            cigar_S_pct = cigar_S_len * 100 / (cigar_M_len + cigar_S_len)

            # for clipping reads with unmapped part >= min_cigar_S
            if (cigar_M_pct >= 0.3) and (cigar_S_pct >= 0.3):
                read_seq_left = read_seq[: int(cigar_splitted[0][:-1])]
                read_seq_right = read_seq[-int(cigar_splitted[1][:-1]):]

                if cigar_splitted[0][-1] == 'M':

                    # write out the sequence of unmapped part
                    clipping_reads_not_matched_part_seq_handle.write('>%s_r\n' % read_id_with_ref_pos)
                    clipping_reads_not_matched_part_seq_handle.write(read_seq_right + '\n')

                    # store the match info of mapped part
                    if ('%s_l' % read_id_with_ref_pos) not in clipping_reads_mapped_part_dict:
                        clipping_reads_mapped_part_dict[('%s_l' % read_id_with_ref_pos)] = [ref_id_with_prefix]
                    else:
                        clipping_reads_mapped_part_dict[('%s_l' % read_id_with_ref_pos)].append(ref_id_with_prefix)

                if cigar_splitted[1][-1] == 'M':

                    # write out the sequence of unmapped part
                    clipping_reads_not_matched_part_seq_handle.write('>%s_l\n' % read_id_with_ref_pos)
                    clipping_reads_not_matched_part_seq_handle.write(read_seq_left + '\n')

                    # store the match info of mapped part
                    if ('%s_r' % read_id_with_ref_pos) not in clipping_reads_mapped_part_dict:
                        clipping_reads_mapped_part_dict[('%s_r' % read_id_with_ref_pos)] = [ref_id_with_prefix]
                    else:
                        clipping_reads_mapped_part_dict[('%s_r' % read_id_with_ref_pos)].append(ref_id_with_prefix)

                clipping_mapped_reads_list.add(read_id)

            # for clipping reads with cigar_S_pct < perfect_match_max_cigar_S_pct
            # treat these reads as perfectly mapped reads
            if (cigar_M_pct >= perfect_match_min_cigar_M_pct) and (cigar_S_pct < perfect_match_max_cigar_S_pct):
                qualified_perfect_match = True


        # if ('S' in cigar) and (len(cigar_splitted) == 3):
        #     if (cigar_splitted[0][-1] == 'S') and (cigar_splitted[1][-1] == 'M') and (cigar_splitted[2][-1] == 'S'):
        #         cigar_M_len = int(cigar_splitted[1][:-1])
        #         cigar_S_len_l = int(cigar_splitted[0][:-1])
        #         cigar_S_len_r = int(cigar_splitted[2][:-1])
        #         cigar_M_pct = cigar_M_len * 100 / (cigar_M_len + cigar_S_len_l + cigar_S_len_r)
        #         cigar_S_pct = (cigar_S_len_l + cigar_S_len_r) * 100 / (cigar_M_len + cigar_S_len_l + cigar_S_len_r)
        #         if (cigar_S_len_l <= 1) or (cigar_S_len_r <= 1):
        #             if (cigar_M_pct >= perfect_match_min_cigar_M_pct) and (cigar_S_pct < perfect_match_max_cigar_S_pct):
        #                 qualified_perfect_match = True


        if ('S' in cigar) and (len(cigar_splitted) == 4):

            if (cigar_splitted[0][-1] == 'S') and (cigar_splitted[1][-1] == 'M') and (cigar_splitted[3][-1] == 'M'):
                cigar_m1_len = int(cigar_splitted[1][:-1])
                cigar_m2_len = int(cigar_splitted[3][:-1])
                mismatch_len = int(cigar_splitted[2][:-1])
                total_M = cigar_m1_len + cigar_m2_len
                error_rate = mismatch_len / (total_M)
                cigar_S_len = int(cigar_splitted[0][:-1])
                cigar_M_pct = total_M * 100 / (total_M + cigar_S_len)
                cigar_S_pct = cigar_S_len * 100 / (total_M + cigar_S_len)
                if (cigar_m1_len >= 30) and (cigar_m2_len >= 30) and (error_rate <= error_rate_cutoff):
                    if (cigar_M_pct >= perfect_match_min_cigar_M_pct) and (cigar_S_pct < perfect_match_max_cigar_S_pct):
                        qualified_perfect_match = True

            if (cigar_splitted[0][-1] == 'M') and (cigar_splitted[2][-1] == 'M') and (cigar_splitted[3][-1] == 'S'):
                cigar_m1_len = int(cigar_splitted[0][:-1])
                cigar_m2_len = int(cigar_splitted[2][:-1])
                mismatch_len = int(cigar_splitted[1][:-1])
                total_M = cigar_m1_len + cigar_m2_len
                error_rate = mismatch_len / (total_M)
                cigar_S_len = int(cigar_splitted[3][:-1])
                cigar_M_pct = total_M * 100 / (total_M + cigar_S_len)
                cigar_S_pct = cigar_S_len * 100 / (total_M + cigar_S_len)
                if (cigar_m1_len >= 30) and (cigar_m2_len >= 30) and (error_rate <= error_rate_cutoff):
                    if (cigar_M_pct >= perfect_match_min_cigar_M_pct) and (cigar_S_pct < perfect_match_max_cigar_S_pct):
                        qualified_perfect_match = True

        # add to dict
        if qualified_perfect_match is True:
            if read_id_base not in perfectly_mapped_reads_dict:
                perfectly_mapped_reads_dict[read_id_base] = {read_strand: [ref_id_with_prefix]}
            else:
                if read_strand not in perfectly_mapped_reads_dict[read_id_base]:
                    perfectly_mapped_reads_dict[read_id_base][read_strand] = [ref_id_with_prefix]
                else:
                    perfectly_mapped_reads_dict[read_id_base][read_strand].append(ref_id_with_prefix)

clipping_reads_not_matched_part_seq_handle.close()

bothed = 0
perfectly_mapped_read_singleton_dict = {}
for perfectly_mapped_read in perfectly_mapped_reads_dict:
    current_value = perfectly_mapped_reads_dict[perfectly_mapped_read]
    if len(current_value) == 1:
        perfectly_mapped_read_singleton_dict[perfectly_mapped_read] = current_value
    if len(current_value) == 2:
        bothed += 1


# get the id of paired reads to extract
solely_perfectly_mapped_reads_r1 = set()
solely_perfectly_mapped_reads_r2 = set()
for perfectly_mapped_read in perfectly_mapped_read_singleton_dict:
    current_value = perfectly_mapped_read_singleton_dict[perfectly_mapped_read]
    strand = list(current_value.keys())[0]
    if strand == '1':
        r2_to_extract = '%s.2' % perfectly_mapped_read
        if r2_to_extract not in all_mapped_reads_set:
        #if r2_to_extract not in clipping_mapped_reads_list:
            solely_perfectly_mapped_reads_r2.add(r2_to_extract)

    elif strand == '2':
        r1_to_extract = '%s.1' % perfectly_mapped_read
        if r1_to_extract not in all_mapped_reads_set:
        #if r1_to_extract not in clipping_mapped_reads_list:
            solely_perfectly_mapped_reads_r1.add(r1_to_extract)


print(len(solely_perfectly_mapped_reads_r1) + len(solely_perfectly_mapped_reads_r2))
# default: 1318
# new with 0.005 error rate: 1504





import os


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def overlap_between_list(list_1, list_2):

    overlap = False
    for list_1_element in list_1:
        if list_1_element in list_2:
            overlap = True

    return overlap


def gnm_level_asessment(linked_mag_dict_rd1):

    rd1_correct_num = 0
    rd1_wrong_gnm = set()
    rd1_unknown_gnm = set()
    rd1_ambiguous_gnm = set()
    for each_rd_1_mag in linked_mag_dict_rd1:
        mag_assess = linked_mag_dict_rd1[each_rd_1_mag]
        if len(mag_assess) == 1:
            if mag_assess == {'Correct'}:
                rd1_correct_num += 1
            elif mag_assess == {'Wrong'}:
                rd1_wrong_gnm.add(each_rd_1_mag)
            elif mag_assess == {'Unknown'}:
                rd1_unknown_gnm.add(each_rd_1_mag)
        elif len(mag_assess) == 2:
            if ('Correct' in mag_assess) and ('Unknown' in mag_assess):
                rd1_correct_num += 1
            if ('Correct' in mag_assess) and ('Wrong' in mag_assess):
                rd1_ambiguous_gnm.add(each_rd_1_mag)
            if ('Wrong' in mag_assess) and ('Unknown' in mag_assess):
                rd1_wrong_gnm.add(each_rd_1_mag)
        elif len(mag_assess) == 3:
            rd1_ambiguous_gnm.add(each_rd_1_mag)

    return rd1_correct_num, rd1_unknown_gnm, rd1_wrong_gnm, rd1_ambiguous_gnm


########################################################################################################################

wd = '/Users/songweizhi/Desktop/assess_linkages_Oral'

########## reference to cluster ##########

drep_ani_cutoff             = 97
drep_cdb_file               = '%s/file_in/Cdb_%s.csv'                                       % (wd, drep_ani_cutoff)
# ref_to_strain_file          = '%s/file_in/ref_to_strain.txt'                                % wd

########## bin to reference ##########

parse_blastn_bin_vs_ref     = False  # True or False
blastn_bin_vs_ref           = '%s/file_in/bin_vs_ref.tab'                                   % wd
iden_cutoff                 = 99.5
aln_len_cutoff              = 1500
cov_q_cutoff                = 90
min_match_length            = 524288  # 100 Kbp  102400
# mag_metadata                = '%s/file_in/MAG_metadata.txt'                                 % wd

########## 16S to reference ##########

perform_blastn_16s_vs_refs  = False  # True or False
combined_GI_ref_16S         = '%s/file_in/combined_Oral_ref_16S.ffn'                     % wd
matam_16s_seqs              = '%s/file_in/CAMI_Oral_138_16S_0.999.polished.fa'           % wd
matam_16s_blastn            = '%s/file_in/CAMI_Oral_138_16S_0.999.polished_vs_ref.tab'   % wd
iden_cutoff_16s             = 99.5  # 99.3 (best), 99.5
aln_len_cutoff_16s          = 500
cov_q_cutoff_16s            = 70
total_query_mag_num         = 87

########## assessment results ##########

MarkerMAG_linkages          = '%s/Oral_0719_45_45_min1500_diff10_linkages_by_genome.txt'             % wd
linkages_from_rd1           = False


# MarkerMAG_linkages          = '/Users/songweizhi/Desktop/tunning_rd1_Oral/stats_combined_filtered.txt'
# MarkerMAG_linkages          = '/Users/songweizhi/Desktop/tunning_rd2_Oral/stats_GapFilling_gnm_filtered.txt'
# linkages_from_rd1           = True

'''
Oral_0719_45_45_min1200_linkages_by_genome.txt	Rd_1	|	59	58	1	0	58/58(100.0)	|	14	14	0	0	0	14/14(100.0)
Oral_0719_45_45_min1200_linkages_by_genome.txt	Rd_2	|	11	10	0	1	10/11(90.91)	|	7	6	0	1	0	6/7(85.71)
Oral_0719_45_45_min1200_linkages_by_genome.txt	Both	|	70	68	1	1	68/69(98.55)	|	21	20	0	1	0	20/21(95.24)

Oral_0719_60_60_min1200_linkages_by_genome.txt	Rd_1	|	43	42	1	0	42/42(100.0)	|	13	13	0	0	0	13/13(100.0)
Oral_0719_60_60_min1200_linkages_by_genome.txt	Rd_2	|	7	6	0	1	6/7(85.71)	|	4	3	0	1	0	3/4(75.0)
Oral_0719_60_60_min1200_linkages_by_genome.txt	Both	|	50	48	1	1	48/49(97.96)	|	17	16	0	1	0	16/17(94.12)

Oral_0719_45_45_min1500_linkages_by_genome.txt	Rd_1	|	46	46	0	0	46/46(100.0)	|	13	13	0	0	0	13/13(100.0)
Oral_0719_45_45_min1500_linkages_by_genome.txt	Rd_2	|	0	0	0	0	0/0(0)	        |	0	0	0	0	0	0/0(0)
Oral_0719_45_45_min1500_linkages_by_genome.txt	Both	|	46	46	0	0	46/46(100.0)	|	13	13	0	0	0	13/13(100.0)

Oral_0719_45_45_min1500_diff10_linkages_by_genome.txt	Rd_1	|	52	52	0	0	52/52(100.0)	|	13	13	0	0	0	13/13(100.0)
Oral_0719_45_45_min1500_diff10_linkages_by_genome.txt	Rd_2	|	0	0	0	0	0/0(0)	        |	0	0	0	0	0	0/0(0)
Oral_0719_45_45_min1500_diff10_linkages_by_genome.txt	Both	|	52	52	0	0	52/52(100.0)	|	13	13	0	0	0	13/13(100.0)

Oral_0719_60_60_min1500_linkages_by_genome.txt	Rd_1	|	36	36	0	0	36/36(100.0)	|	12	12	0	0	0	12/12(100.0)
Oral_0719_60_60_min1500_linkages_by_genome.txt	Rd_2	|	8	8	0	0	8/8(100.0)	    |	5	5	0	0	0	5/5(100.0)
Oral_0719_60_60_min1500_linkages_by_genome.txt	Both	|	44	44	0	0	44/44(100.0)	|	17	17	0	0	0	17/17(100.0)





Oral_0718_45_45_min1200_linkages_by_genome.txt	Rd_1	|	60	59	1	0	59/59(100.0)	|	15	15	0	0	0	15/15(100.0)
Oral_0718_45_45_min1200_linkages_by_genome.txt	Rd_2	|	11	10	0	1	10/11(90.91)	|	7	6	0	1	0	6/7(85.71)
Oral_0718_45_45_min1200_linkages_by_genome.txt	Both	|	71	69	1	1	69/70(98.57)	|	22	21	0	1	0	21/22(95.45)

Oral_0718_60_60_min1500_linkages_by_genome.txt	Rd_1	|	37	37	0	0	37/37(100.0)	|	13	13	0	0	0	13/13(100.0)
Oral_0718_60_60_min1500_linkages_by_genome.txt	Rd_2	|	8	5	0	3	5/8(62.5)	|	5	4	0	1	0	4/5(80.0)
Oral_0718_60_60_min1500_linkages_by_genome.txt	Both	|	45	42	0	3	42/45(93.33)	|	18	17	0	1	0	17/18(94.44)

Oral_0717_45_45_min1200_linkages_by_genome.txt	Rd_1	|	60	59	1	0	59/59(100.0)	|	15	15	0	0	0	15/15(100.0)
Oral_0717_45_45_min1200_linkages_by_genome.txt	Rd_2	|	12	10	0	2	10/12(83.33)	|	8	6	0	2	0	6/8(75.0)
Oral_0717_45_45_min1200_linkages_by_genome.txt	Both	|	72	69	1	2	69/71(97.18)	|	23	21	0	2	0	21/23(91.3)

Oral_0715_45_45_min1200_linkages_by_genome.txt	Rd_1	|	60	59	1	0	59/59(100.0)	|	15	15	0	0	0	15/15(100.0)
Oral_0715_45_45_min1200_linkages_by_genome.txt	Rd_2	|	12	10	0	2	10/12(83.33)	|	8	6	0	2	0	7/8(87.5)
Oral_0715_45_45_min1200_linkages_by_genome.txt	Both	|	72	69	1	2	69/71(97.18)	|	23	21	0	2	0	22/23(95.7)

Oral_0715_60_60_min1200_linkages_by_genome.txt	Rd_1	|	44	43	1	0	43/43(100.0)	|	14	14	0	0	0	14/14(100.0)
Oral_0715_60_60_min1200_linkages_by_genome.txt	Rd_2	|	8	7	0	1	7/8(87.5)	    |	5	4	0	1	0	5/5(100.0)
Oral_0715_60_60_min1200_linkages_by_genome.txt	Both	|	52	50	1	1	50/51(98.04)	|	19	18	0	1	0	19/19(100.0)

Oral_0715_45_45_min1200_linkages_by_genome.txt	Rd_1	|	60	59	1	0	59/59(100.0)	|	15	15	0	0	0	15/15(100.0)
Oral_0715_45_45_min1200_linkages_by_genome.txt	Rd_2	|	12	10	0	2	10/12(83.33)	|	8	6	0	2	0	6/8(75.0)
Oral_0715_45_45_min1200_linkages_by_genome.txt	Both	|	72	69	1	2	69/71(97.18)	|	23	21	0	2	0	21/23(91.3)

Oral_0715_60_60_min1200_linkages_by_genome.txt	Rd_1	|	44	43	1	0	43/43(100.0)	|	14	14	0	0	0	14/14(100.0)
Oral_0715_60_60_min1200_linkages_by_genome.txt	Rd_2	|	8	7	0	1	7/8(87.5)	    |	5	4	0	1	0	4/5(80.0)
Oral_0715_60_60_min1200_linkages_by_genome.txt	Both	|	52	50	1	1	50/51(98.04)	|	19	18	0	1	0	18/19(94.74)

Oral_0715_60_60_min1500_linkages_by_genome.txt	Rd_1	|	37	37	0	0	37/37(100.0)	|	13	13	0	0	0	13/13(100.0)
Oral_0715_60_60_min1500_linkages_by_genome.txt	Rd_2	|	8	5	0	3	5/8(62.5)	    |	5	4	0	1	0	4/5(80.0)
Oral_0715_60_60_min1500_linkages_by_genome.txt	Both	|	45	42	0	3	42/45(93.33)	|	18	17	0	1	0	17/18(94.44)

good!!!
Oral_0713_60_60_min1500_linkages_by_genome.txt	Rd_1	|	45	40	0	5	40/45(88.89)	|	15	14	0	1	0	14/15(93.33)
Oral_0713_60_60_min1500_linkages_by_genome.txt	Rd_2	|	10	10	0	0	10/10(100.0)	|	3	3	0	0	0	3/3(100.0)
Oral_0713_60_60_min1500_linkages_by_genome.txt	Both	|	55	50	0	5	50/55(90.91)	|	18	17	0	1	0	17/18(94.44)
good!!!
Oral_0713_60_60_min1200_linkages_by_genome.txt	Rd_1	|	49	43	1	5	43/48(89.58)	|	15	14	0	1	0	14/15(93.33)
Oral_0713_60_60_min1200_linkages_by_genome.txt	Rd_2	|	4	3	0	1	3/4(75.0)	    |	3	2	0	1	0	2/3(66.67)
Oral_0713_60_60_min1200_linkages_by_genome.txt	Both	|	53	46	1	6	46/52(88.46)	|	18	16	0	2	0	16/18(88.89)

Oral_0713_45_45_min1200_linkages_by_genome.txt	Rd_1	|	93	81	1	11	81/92(88.04)	|	22	18	0	4	0	18/22(81.82)
Oral_0713_45_45_min1200_linkages_by_genome.txt	Rd_2	|	15	5	0	10	5/15(33.33)	    |	7	4	0	3	0	4/7(57.14)
Oral_0713_45_45_min1200_linkages_by_genome.txt	Both	|	108	86	1	21	86/107(80.37)	|	29	22	0	7	0	22/29(75.86)

Oral_0708_60_60_min1200_mask_linkages_by_genome.txt	Rd_1	|	50	43	1	6	43/49(87.76)	|	16	14	0	2	0	14/16(87.5)
Oral_0708_60_60_min1200_mask_linkages_by_genome.txt	Rd_2	|	16	12	0	4	12/16(75.0)	    |	4	2	0	2	0	2/4(50.0)
Oral_0708_60_60_min1200_mask_linkages_by_genome.txt	Both	|	66	55	1	10	55/65(84.62)	|	20	16	0	4	0	16/20(80.0)
Oral_0708_60_60_min1500_mask_linkages_by_genome.txt	Rd_1	|	46	40	0	6	40/46(86.96)	|	16	14	0	2	0	14/16(87.5)
Oral_0708_60_60_min1500_mask_linkages_by_genome.txt	Rd_2	|	9	1	0	8	1/9(11.11)	    |	2	1	0	1	0	1/2(50.0)
Oral_0708_60_60_min1500_mask_linkages_by_genome.txt	Both	|	55	41	0	14	41/55(74.55)	|	18	15	0	3	0	15/18(83.33)
Oral_0705_60_60_min1500_linkages_by_genome.txt	Rd_1	|	45	42	0	3	42/45(93.33)	|	16	15	0	1	0	15/16(93.75)
Oral_0705_60_60_min1500_linkages_by_genome.txt	Rd_2	|	5	5	0	0	5/5(100.0)	    |	2	2	0	0	0	2/2(100.0)
Oral_0705_60_60_min1500_linkages_by_genome.txt	Both	|	50	47	0	3	47/50(94.0)	    |	18	17	0	1	0	17/18(94.44)
Oral_0705_60_60_min1400_linkages_by_genome.txt	Rd_1	|	38	37	0	1	37/38(97.37)	|	15	14	0	1	0	14/15(93.33)
Oral_0705_60_60_min1400_linkages_by_genome.txt	Rd_2	|	17	1	0	16	1/17(5.88)	    |	3	0	0	3	0	0/3(0.0)
Oral_0705_60_60_min1400_linkages_by_genome.txt	Both	|	55	38	0	17	38/55(69.09)	|	18	14	0	4	0	14/18(77.78)
Oral_0622_60_60_polish_min1200_99_127_linkages_by_genome.txt	Rd_1	|	44	39	1	4	39/43(90.7)	    |	16	15	0	1	0	15/16(93.75)
Oral_0622_60_60_polish_min1200_99_127_linkages_by_genome.txt	Rd_2	|	20	6	0	14	6/20(30.0)	    |	3	1	0	2	0	1/3(33.33)
Oral_0622_60_60_polish_min1200_99_127_linkages_by_genome.txt	Both	|	64	45	1	18	45/63(71.43)	|	19	16	0	3	0	16/19(84.21)
Oral_0622_60_60_polish_new_linkages_by_genome.txt	Rd_1	|	46	44	1	1	44/45(97.78)	|	16	15	0	1	0	15/16(93.75)
Oral_0622_60_60_polish_new_linkages_by_genome.txt	Rd_2	|	23	9	0	14	9/23(39.13)	    |	4	2	0	2	0	2/4(50.0)
Oral_0622_60_60_polish_new_linkages_by_genome.txt	Both	|	69	53	1	15	53/68(77.94)	|	20	17	0	3	0	17/20(85.0)
Oral_0622_60_60_polish_min1200_75_127_linkages_by_genome.txt	Rd_1	|	45	43	1	1	43/44(97.73)	|	16	15	0	1	0	15/16(93.75)
Oral_0622_60_60_polish_min1200_75_127_linkages_by_genome.txt	Rd_2	|	23	7	0	16	7/23(30.43)	    |	4	1	0	3	0	1/4(25.0)
Oral_0622_60_60_polish_min1200_75_127_linkages_by_genome.txt	Both	|	68	50	1	17	50/67(74.63)	|	20	16	0	4	0	16/20(80.0)
Oral_0622_60_60_polish_min1200_linkages_by_genome.txt	Rd_1	|	50	44	1	5	44/49(89.8)	    |	16	15	0	1	0	15/16(93.75)
Oral_0622_60_60_polish_min1200_linkages_by_genome.txt	Rd_2	|	14	11	0	3	11/14(78.57)	|	3	1	0	2	0	1/3(33.33)
Oral_0622_60_60_polish_min1200_linkages_by_genome.txt	Both	|	64	55	1	8	55/63(87.3)	    |	19	16	0	3	0	16/19(84.21)
Oral_0622_60_60_polish_min1000_linkages_by_genome.txt	Rd_1	|	48	42	1	5	42/47(89.36)	|	16	15	0	1	0	15/16(93.75)
Oral_0622_60_60_polish_min1000_linkages_by_genome.txt	Rd_2	|	25	8	0	17	8/25(32.0)	    |	5	2	0	3	0	2/5(40.0)
Oral_0622_60_60_polish_min1000_linkages_by_genome.txt	Both	|	73	50	1	22	50/72(69.44)	|	21	17	0	4	0	17/21(80.95)
Oral_0622_60_60_polish_min900_linkages_by_genome.txt	Rd_1	|	48	45	1	2	45/47(95.74)	|	16	14	0	1	1	14/16(87.5)
Oral_0622_60_60_polish_min900_linkages_by_genome.txt	Rd_2	|	33	14	0	19	14/33(42.42)	|	7	3	0	4	0	3/7(42.86)
Oral_0622_60_60_polish_min900_linkages_by_genome.txt	Both	|	81	59	1	21	59/80(73.75)	|	23	17	0	5	1	17/23(73.91)
Oral_0622_60_60_polish_linkages_by_genome.txt	Rd_1	|	63	52	8	3	52/55(94.55)	|	20	16	1	2	1	16/19(84.21)
Oral_0622_60_60_polish_linkages_by_genome.txt	Rd_2	|	14	10	0	4	10/14(71.43)	|	6	3	0	3	0	3/6(50.0)
Oral_0622_60_60_polish_linkages_by_genome.txt	Both	|	77	62	8	7	62/69(89.86)	|	26	19	1	5	1	19/25(76.0)

'''

########## script ##########

pwd_plot_sankey_R = '%s/file_in/get_sankey_plot.R' % wd

############################################### define file/folder name ################################################

# bin to reference
bin_vs_ref_txt              = '%s/bin_vs_ref_imag%s.txt'                    % (wd, iden_cutoff)
stats_bin_to_ref_txt        = '%s/stats_bin_to_ref_imag%s.txt'              % (wd, iden_cutoff)
stats_ref_to_bin_txt        = '%s/stats_ref_to_bin_imag%s.txt'              % (wd, iden_cutoff)
stats_bin_to_cluster_txt    = '%s/stats_bin_to_cluster_ani%s_imag%s.txt'    % (wd, drep_ani_cutoff, iden_cutoff)
stats_cluster_to_bin_txt    = '%s/stats_cluster_to_bin_ani%s_imag%s.txt'    % (wd, drep_ani_cutoff, iden_cutoff)

# assessment results
linkage_file_path, linkage_file_basename, linkage_file_extension = sep_path_basename_ext(MarkerMAG_linkages)
MarkerMAG_linkages_assessed = '%s/%s_with_assessment_ani%s_imag%s_i16S%s.txt'   % (linkage_file_path, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)
wrong_linkages_txt          = '%s/%s_wrong_ani%s_imag%s_i16S%s.txt'             % (linkage_file_path, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)
unknown_linkages_txt        = '%s/%s_unknown_ani%s_imag%s_i16S%s.txt'           % (linkage_file_path, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)


################################################# reference to cluster #################################################

cluster_to_ref_dict = {}
ref_to_cluster_dict = {}
for each_ref in open(drep_cdb_file):
    if not each_ref.startswith('genome,secondary_cluster'):
        each_ref_split = each_ref.strip().split(',')
        ref_file_name = each_ref_split[0]
        ref_file_name_no_ext = '.'.join(ref_file_name.split('.')[:-1])
        ref_cluster = 'C' + each_ref_split[1]
        ref_to_cluster_dict[ref_file_name_no_ext] = ref_cluster
        if ref_cluster not in cluster_to_ref_dict:
            cluster_to_ref_dict[ref_cluster] = [ref_file_name_no_ext]
        else:
            cluster_to_ref_dict[ref_cluster].append(ref_file_name_no_ext)


################################################### bin to reference ###################################################

bin_ref_connector           = '__|__'

# get ref_to_strain_dict
ref_to_strain_dict = {}
# for ref in open(ref_to_strain_file):
#     ref_split = ref.strip().split('\t')
#     ref_to_strain_dict[ref_split[0]] = ref_split[1]

if parse_blastn_bin_vs_ref is True:
    bin_vs_ref_dict = {}
    for match in open(blastn_bin_vs_ref):
        match_split = match.strip().split('\t')
        query = match_split[0]
        query_genome = '_'.join(query.split('_')[:2])
        subject = match_split[1]
        subject_genome = '_'.join(subject.split('_')[:2])
        iden = float(match_split[2])
        aln_len = int(match_split[3])
        query_len = int(match_split[12])
        subject_len = int(match_split[13])
        coverage_q = float(aln_len) * 100 / float(query_len)
        if (iden >= iden_cutoff) and (aln_len > aln_len_cutoff) and (coverage_q >= cov_q_cutoff):
            key_bin_ref = '%s%s%s' % (query_genome, bin_ref_connector, subject_genome)
            if key_bin_ref not in bin_vs_ref_dict:
                bin_vs_ref_dict[key_bin_ref] = aln_len
            else:
                bin_vs_ref_dict[key_bin_ref] += aln_len

    bin_vs_ref_dict_filtered = {}
    for each_key in bin_vs_ref_dict:
        if bin_vs_ref_dict[each_key] >= min_match_length:
            bin_vs_ref_dict_filtered[each_key] = bin_vs_ref_dict[each_key]

    # write out linkages
    linkage_txt_handle = open(bin_vs_ref_txt, 'w')
    linkage_txt_handle.write('Bin,Ref,Length\n')
    bin_to_ref_dict = {}
    ref_to_bin_dict = {}
    bin_to_cluster_dict = {}
    cluster_to_bin_dict = {}
    for each_linkage in bin_vs_ref_dict_filtered:
        linkage_split = each_linkage.split(bin_ref_connector)
        bin_id = linkage_split[0]
        ref_id = linkage_split[1]
        ref_cluster = ref_to_cluster_dict[ref_id]

        if bin_id not in bin_to_ref_dict:
            bin_to_ref_dict[bin_id] = {ref_id}
        else:
            bin_to_ref_dict[bin_id].add(ref_id)

        if bin_id not in bin_to_cluster_dict:
            bin_to_cluster_dict[bin_id] = {ref_cluster}
        else:
            bin_to_cluster_dict[bin_id].add(ref_cluster)

        if ref_id not in ref_to_bin_dict:
            ref_to_bin_dict[ref_id] = {bin_id}
        else:
            ref_to_bin_dict[ref_id].add(bin_id)

        if ref_cluster not in cluster_to_bin_dict:
            cluster_to_bin_dict[ref_cluster] = {bin_id}
        else:
            cluster_to_bin_dict[ref_cluster].add(bin_id)

        linkage_txt_handle.write('%s,%s,%s\n' % (bin_id, ref_id, bin_vs_ref_dict_filtered[each_linkage]))
    linkage_txt_handle.close()

    # visualize
    cmd_sankey_bin_to_ref = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, bin_vs_ref_txt, 600, 1800)
    os.system(cmd_sankey_bin_to_ref)

    bin_to_ref_txt_handle = open(stats_bin_to_ref_txt, 'w')
    for each_bin in bin_to_ref_dict:
        bin_to_ref_txt_handle.write('%s\t%s\n' % (each_bin, ','.join(bin_to_ref_dict[each_bin])))
    bin_to_ref_txt_handle.close()

    ref_to_bin_txt_handle = open(stats_ref_to_bin_txt, 'w')
    for each_ref in ref_to_bin_dict:
        ref_to_bin_txt_handle.write('%s\t%s\t%s\n' % (each_ref, ','.join(ref_to_bin_dict[each_ref]), ref_to_strain_dict.get(each_ref, 'NA')))
    ref_to_bin_txt_handle.close()

    stats_bin_to_cluster_txt_handle = open(stats_bin_to_cluster_txt, 'w')
    for each_bin in bin_to_cluster_dict:
        stats_bin_to_cluster_txt_handle.write('%s\t%s\n' % (each_bin, ','.join(bin_to_cluster_dict[each_bin])))
    stats_bin_to_cluster_txt_handle.close()

    stats_cluster_to_bin_txt_handle = open(stats_cluster_to_bin_txt, 'w')
    for each_ref in cluster_to_bin_dict:
        stats_cluster_to_bin_txt_handle.write('%s\t%s\n' % (each_ref, ','.join(cluster_to_bin_dict[each_ref])))
    stats_cluster_to_bin_txt_handle.close()

bin_to_cluster_dict = {}
for each_match in open(stats_bin_to_cluster_txt):
    each_match_split = each_match.strip().split('\t')
    bin_to_cluster_dict[each_match_split[0]] = {i for i in each_match_split[1].split(',')}


cluster_to_bin_dict = {}
for each_bin in bin_to_cluster_dict:
    matched_clusters =  bin_to_cluster_dict[each_bin]
    for matched_cluster in matched_clusters:
        if matched_cluster not in cluster_to_bin_dict:
            cluster_to_bin_dict[matched_cluster] = {each_bin}
        else:
            cluster_to_bin_dict[matched_cluster].add(each_bin)


################################################### 16S to reference ###################################################

blast_parameters = '-evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 1'
blast_cmd = 'blastn -query %s -subject %s -out %s %s' % (matam_16s_seqs, combined_GI_ref_16S, matam_16s_blastn, blast_parameters)
if perform_blastn_16s_vs_refs is True:
    os.system(blast_cmd)

# get matam_16s_to_cluster_dict
matam_16s_to_cluster_dict = {}
for match in open(matam_16s_blastn):
    match_split = match.strip().split('\t')
    query = match_split[0]
    subject = match_split[1]
    subject_gnm = subject.split('_16S_')[0]
    subject_cluster = ref_to_cluster_dict[subject_gnm]
    iden = float(match_split[2])
    aln_len = int(match_split[3])
    query_len = int(match_split[12])
    subject_len = int(match_split[13])
    coverage_q = float(aln_len) * 100 / float(query_len)
    if (iden >= iden_cutoff_16s) and (aln_len > aln_len_cutoff_16s) and (coverage_q >= cov_q_cutoff_16s):
        if query not in matam_16s_to_cluster_dict:
            matam_16s_to_cluster_dict[query] = {subject_cluster}
        else:
            matam_16s_to_cluster_dict[query].add(subject_cluster)

cluster_to_matam_16s_dict = {}
for matam_16s in matam_16s_to_cluster_dict:
    matched_clusters =  matam_16s_to_cluster_dict[matam_16s]
    for matched_cluster in matched_clusters:
        if matched_cluster not in cluster_to_matam_16s_dict:
            cluster_to_matam_16s_dict[matched_cluster] = {matam_16s}
        else:
            cluster_to_matam_16s_dict[matched_cluster].add(matam_16s)


###################################################### assessment ######################################################

MarkerMAG_linkages_assessed_handle = open(MarkerMAG_linkages_assessed, 'w')
wrong_linkages_txt_handle = open(wrong_linkages_txt, 'w')
unknown_linkages_txt_handle = open(unknown_linkages_txt, 'w')
linkage_num_right = 0
linkage_num_wrong = 0
linkage_num_unknown = 0
linkage_assessment_dict = {}
for each_linkage in open(MarkerMAG_linkages):
    if ('MarkerGene\tGenomicSeq\tLinkage\tRound' in each_linkage) or ('MarkerGene,GenomicSeq,Number' in each_linkage):
        MarkerMAG_linkages_assessed_handle.write('MarkerGene\tGenomicSeq\tLinkage\tStep\tAssessment\n')
    else:
        id_16s = ''
        id_mag = ''
        link_num = ''
        if linkages_from_rd1 is False:
            each_linkage_split = each_linkage.strip().split('\t')
            id_16s = each_linkage_split[0]
            id_mag = each_linkage_split[1]
            link_num = each_linkage_split[2]
        else:
            each_linkage_split = each_linkage.strip().split(',')
            id_16s = each_linkage_split[0][12:]
            id_mag = each_linkage_split[1][12:]
            link_num = each_linkage_split[2]

        key_16s_mag = '%s___%s' % (id_16s, id_mag)

        matched_cluster_16s = {}
        if id_16s in matam_16s_to_cluster_dict:
            matched_cluster_16s = matam_16s_to_cluster_dict[id_16s]

        matched_cluster_mag = {}
        if id_mag in bin_to_cluster_dict:
            matched_cluster_mag = bin_to_cluster_dict[id_mag]

        if (matched_cluster_16s != {}) and (matched_cluster_mag != {}):

            if overlap_between_list(matched_cluster_mag, matched_cluster_16s) is True:
                linkage_num_right += 1

                if linkages_from_rd1 is False:
                    MarkerMAG_linkages_assessed_handle.write('%s\tCorrect\n' % each_linkage.strip())
                else:
                    MarkerMAG_linkages_assessed_handle.write('%s\t%s\t%s\tRd1\tCorrect\n' % (id_16s, id_mag, link_num))

                if id_mag not in linkage_assessment_dict:
                    linkage_assessment_dict[id_mag] = ['Correct']
                else:
                    linkage_assessment_dict[id_mag].append('Correct')

            else:
                linkage_num_wrong += 1

                if linkages_from_rd1 is False:
                    MarkerMAG_linkages_assessed_handle.write('%s\tWrong\n' % each_linkage.strip())
                else:
                    MarkerMAG_linkages_assessed_handle.write('%s\t%s\t%s\tRd1\tWrong\n' % (id_16s, id_mag, link_num))

                if id_mag not in linkage_assessment_dict:
                    linkage_assessment_dict[id_mag] = ['Wrong']
                else:
                    linkage_assessment_dict[id_mag].append('Wrong')

                for each_16s_cluster in matched_cluster_16s:
                    current_16s_cluster_ref = cluster_to_ref_dict[each_16s_cluster]
                    for each_ref_1 in current_16s_cluster_ref:
                        wrong_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_wrong, id_16s, id_mag, '16S', each_16s_cluster, each_ref_1, ref_to_strain_dict.get(each_ref_1, 'NA')))

                for each_mag_cluster in matched_cluster_mag:
                    current_mag_cluster_ref = cluster_to_ref_dict[each_mag_cluster]
                    for each_ref_2 in current_mag_cluster_ref:
                        wrong_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_wrong, id_16s, id_mag, 'MAG', each_mag_cluster, each_ref_2, ref_to_strain_dict.get(each_ref_2, 'NA')))
                wrong_linkages_txt_handle.write('\n')
        else:
            linkage_num_unknown += 1

            if linkages_from_rd1 is False:
                MarkerMAG_linkages_assessed_handle.write('%s\tUnknown\n' % each_linkage.strip())
            else:
                MarkerMAG_linkages_assessed_handle.write('%s\t%s\t%s\tRd1\tUnknown\n' % (id_16s, id_mag, link_num))

            if id_mag not in linkage_assessment_dict:
                linkage_assessment_dict[id_mag] = ['Unknown']
            else:
                linkage_assessment_dict[id_mag].append('Unknown')

            for each_16s_cluster in matched_cluster_16s:
                current_16s_cluster_ref = cluster_to_ref_dict[each_16s_cluster]
                for each_ref_1 in current_16s_cluster_ref:
                    unknown_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_unknown, id_16s, id_mag, '16S', each_16s_cluster, each_ref_1, ref_to_strain_dict.get(each_ref_1, 'NA')))
            for each_mag_cluster in matched_cluster_mag:
                current_mag_cluster_ref = cluster_to_ref_dict[each_mag_cluster]
                for each_ref_2 in current_mag_cluster_ref:
                    unknown_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_unknown, id_16s, id_mag, 'MAG', each_mag_cluster, each_ref_2, ref_to_strain_dict.get(each_ref_2, 'NA')))
            unknown_linkages_txt_handle.write('\n')
wrong_linkages_txt_handle.close()
unknown_linkages_txt_handle.close()
MarkerMAG_linkages_assessed_handle.close()


# get assess stats at mag level
linked_mag_dict_rd1 = {}
linked_mag_dict_rd2 = {}
total_linkage_num_rd1 = 0
total_linkage_num_rd2 = 0
correct_link_rd1 = 0
correct_link_rd2 = 0
wrong_link_rd1 = 0
wrong_link_rd2 = 0
unknown_link_rd1 = 0
unknown_link_rd2 = 0
for each_link in open(MarkerMAG_linkages_assessed):
    if not each_link.startswith('MarkerGene	GenomicSeq	Linkage	Step	Assessment'):
        each_link_split = each_link.strip().split('\t')
        gnm_id = each_link_split[1]
        linked_rd = each_link_split[3]
        assessment = each_link_split[4]
        if linked_rd == 'Rd1':
            total_linkage_num_rd1 += 1

            if assessment == 'Correct':
                correct_link_rd1 += 1
            if assessment == 'Wrong':
                wrong_link_rd1 += 1
            if assessment == 'Unknown':
                unknown_link_rd1 += 1

            if gnm_id not in linked_mag_dict_rd1:
                linked_mag_dict_rd1[gnm_id] = {assessment}
            else:
                linked_mag_dict_rd1[gnm_id].add(assessment)
        if linked_rd == 'Rd2':
            total_linkage_num_rd2 += 1

            if assessment == 'Correct':
                correct_link_rd2 += 1
            if assessment == 'Wrong':
                wrong_link_rd2 += 1
            if assessment == 'Unknown':
                unknown_link_rd2 += 1

            if gnm_id not in linked_mag_dict_rd1:
                linked_mag_dict_rd2[gnm_id] = {assessment}
            else:
                linked_mag_dict_rd2[gnm_id].add(assessment)


total_linkage_num_both = total_linkage_num_rd1 + total_linkage_num_rd2
total_linked_mag_num = len(linked_mag_dict_rd1) + len(linked_mag_dict_rd2)

rd1_correct_num, rd1_unknown_gnm, rd1_wrong_gnm, rd1_ambiguous_gnm = gnm_level_asessment(linked_mag_dict_rd1)
rd2_correct_num, rd2_unknown_gnm, rd2_wrong_gnm, rd2_ambiguous_gnm = gnm_level_asessment(linked_mag_dict_rd2)

correct_num_both        = rd1_correct_num + rd2_correct_num
unknown_gnm_both_num    = len(rd1_unknown_gnm) + len(rd2_unknown_gnm)
wrong_gnm_both_num      = len(rd1_wrong_gnm) + len(rd2_wrong_gnm)
ambiguous_gnm_both_num  = len(rd1_ambiguous_gnm) + len(rd2_ambiguous_gnm)

recovery_str_rd1   = '%s/%s(%s)' % (rd1_correct_num,  total_query_mag_num, float("{0:.2f}".format(rd1_correct_num*100/total_query_mag_num)))
recovery_str_rd2   = '%s/%s(%s)' % (rd2_correct_num,  total_query_mag_num, float("{0:.2f}".format(rd2_correct_num*100/total_query_mag_num)))
recovery_str_both  = '%s/%s(%s)' % (correct_num_both, total_query_mag_num, float("{0:.2f}".format(correct_num_both*100/total_query_mag_num)))
accuracy_str_rd1   = '%s/%s(%s)' % (rd1_correct_num,  (len(linked_mag_dict_rd1) - len(rd1_unknown_gnm)), float("{0:.2f}".format(rd1_correct_num*100/(len(linked_mag_dict_rd1) - len(rd1_unknown_gnm)))))
accuracy_str_rd2 = '0/0(0)'
if (len(linked_mag_dict_rd2) - len(rd2_unknown_gnm)) > 0:
    accuracy_str_rd2   = '%s/%s(%s)' % (rd2_correct_num,  (len(linked_mag_dict_rd2) - len(rd2_unknown_gnm)), float("{0:.2f}".format(rd2_correct_num*100/(len(linked_mag_dict_rd2) - len(rd2_unknown_gnm)))))
accuracy_str_both  = '%s/%s(%s)' % (correct_num_both, (total_linked_mag_num - unknown_gnm_both_num), float("{0:.2f}".format(correct_num_both*100/(total_linked_mag_num - unknown_gnm_both_num))))

correct_link_both = correct_link_rd1 + correct_link_rd2
wrong_link_both   = wrong_link_rd1 + wrong_link_rd2
unknown_link_both = unknown_link_rd1 + unknown_link_rd2

accuracy_str_rd1_link_level   = '%s/%s(%s)' % (correct_link_rd1,  (total_linkage_num_rd1 - unknown_link_rd1), float("{0:.2f}".format(correct_link_rd1*100/(total_linkage_num_rd1 - unknown_link_rd1))))
accuracy_str_rd2_link_level = '0/0(0)'
if (total_linkage_num_rd2 - unknown_link_rd2) > 0:
    accuracy_str_rd2_link_level   = '%s/%s(%s)' % (correct_link_rd2,  (total_linkage_num_rd2 - unknown_link_rd2), float("{0:.2f}".format(correct_link_rd2*100/(total_linkage_num_rd2 - unknown_link_rd2))))
accuracy_str_both_link_level  = '%s/%s(%s)' % (correct_link_both, (total_linkage_num_both - unknown_link_both), float("{0:.2f}".format(correct_link_both*100/(total_linkage_num_both - unknown_link_both))))

prefix = os.path.basename(MarkerMAG_linkages).split('_identified_linkages_genome_level')[0]
print('%s\tRound\t|\tLink\tYes\tNA\tNo\tAccuracy\t|\tMAG\tYes\tNA\tNo\tY/N\tAccuracy' % prefix)
print('%s\tRd_1\t|\t%s\t%s\t%s\t%s\t%s\t|\t%s\t%s\t%s\t%s\t%s\t%s' % (prefix, total_linkage_num_rd1,  correct_link_rd1,  unknown_link_rd1,  wrong_link_rd1,  accuracy_str_rd1_link_level,  len(linked_mag_dict_rd1), rd1_correct_num,  len(rd1_unknown_gnm), len(rd1_wrong_gnm), len(rd1_ambiguous_gnm), accuracy_str_rd1))
print('%s\tRd_2\t|\t%s\t%s\t%s\t%s\t%s\t|\t%s\t%s\t%s\t%s\t%s\t%s' % (prefix, total_linkage_num_rd2,  correct_link_rd2,  unknown_link_rd2,  wrong_link_rd2,  accuracy_str_rd2_link_level,  len(linked_mag_dict_rd2), rd2_correct_num,  len(rd2_unknown_gnm), len(rd2_wrong_gnm), len(rd2_ambiguous_gnm), accuracy_str_rd2))
print('%s\tBoth\t|\t%s\t%s\t%s\t%s\t%s\t|\t%s\t%s\t%s\t%s\t%s\t%s' % (prefix, total_linkage_num_both, correct_link_both, unknown_link_both, wrong_link_both, accuracy_str_both_link_level, total_linked_mag_num,      correct_num_both, unknown_gnm_both_num, wrong_gnm_both_num, ambiguous_gnm_both_num, accuracy_str_both))


########################################################################################################################

'''

########################################################################################################################

CAMI_Oral_subsample_50_1551	Oral_57	2123	Rd1	Wrong

CAMI_Oral_subsample_50_1551     C125_1,C125_3,C126_1,C126_2,C126_3
Oral_57                         C126_5

# blast between 16S from Oral_57 ref gnm and Matam assemblies
OTU_97.23667.1_16S	CAMI_Oral_subsample_50_571	100.000	1542	0	0	1	1542	5	1546	0.0	2848

# blast between CAMI_Oral_subsample_50_1551 and 16S from ref gnms

BioSAK iTOL -ColorRange -lg leaf_group_raw.txt -lt Identity -out leaf_group.txt 

########################################################################################################################

'''
# cluster_id = 'C161_2'
# print('cluster to ref  : %s\t%s\t%s' % (cluster_id, len(cluster_to_ref_dict.get(cluster_id, 'NA')), cluster_to_ref_dict.get(cluster_id, 'NA')))
# print('cluster to MAG  : %s\t%s\t%s' % (cluster_id, len(cluster_to_bin_dict.get(cluster_id, 'NA')), cluster_to_bin_dict.get(cluster_id, 'NA')))
# print('cluster to Matam: %s\t%s\t%s' % (cluster_id, len(cluster_to_matam_16s_dict.get(cluster_id, 'NA')), cluster_to_matam_16s_dict.get(cluster_id, 'NA')))
# #print('cluster to MAG: %s\t%s' % ('C126_5', cluster_to_bin_dict.get('C126_5', 'NA')))

#
# print('cluster to MAG  : %s\t%s\t%s' % ('C126_1', len(cluster_to_bin_dict.get('C126_1', 'NA')), cluster_to_bin_dict.get('C126_1', 'NA')))
# print('cluster to MAG  : %s\t%s\t%s' % ('C126_2', len(cluster_to_bin_dict.get('C126_2', 'NA')), cluster_to_bin_dict.get('C126_2', 'NA')))
# print('cluster to MAG  : %s\t%s\t%s' % ('C126_3', len(cluster_to_bin_dict.get('C126_3', 'NA')), cluster_to_bin_dict.get('C126_3', 'NA')))
# print('cluster to MAG  : %s\t%s\t%s' % ('C126_4', len(cluster_to_bin_dict.get('C126_4', 'NA')), cluster_to_bin_dict.get('C126_4', 'NA')))
#


# print('%s\t%s' % ('CAMI_Oral_subsample_50_735', matam_16s_to_cluster_dict['CAMI_Oral_subsample_50_735']))
# print('%s\t%s' % ('CAMI_Oral_subsample_50_818', matam_16s_to_cluster_dict['CAMI_Oral_subsample_50_818']))
# print('%s\t%s' % ('CAMI_Oral_subsample_50_734', matam_16s_to_cluster_dict['CAMI_Oral_subsample_50_734']))
# print('%s\t%s' % ('CAMI_Oral_subsample_10_308', matam_16s_to_cluster_dict['CAMI_Oral_subsample_10_308']))
# print('%s\t%s' % ('CAMI_Oral_subsample_50_279', matam_16s_to_cluster_dict['CAMI_Oral_subsample_50_279']))

# print('%s\t%s' % ('CAMI_Oral_subsample_50_894', matam_16s_to_cluster_dict['CAMI_Oral_subsample_50_894']))
# print('%s\t%s' % ('CAMI_Oral_subsample_50_884', matam_16s_to_cluster_dict['CAMI_Oral_subsample_50_884']))
# print('%s\t%s' % ('CAMI_Oral_subsample_25_465', matam_16s_to_cluster_dict['CAMI_Oral_subsample_25_465']))
# print(bin_to_cluster_dict['Oral_28'])
# print(bin_to_cluster_dict['Oral_42'])
# print(bin_to_cluster_dict['Oral_43'])
# print(bin_to_cluster_dict['Oral_44'])
# print(bin_to_cluster_dict['Oral_46'])
# print(bin_to_cluster_dict['Oral_47'])
# print(bin_to_cluster_dict['Oral_53'])
# print(bin_to_cluster_dict['Oral_57'])
# print(bin_to_cluster_dict['Oral_65'])
# print(bin_to_cluster_dict['Oral_70'])
# print(bin_to_cluster_dict['Oral_72'])
# print(bin_to_cluster_dict['Oral_75'])

'''
CAMI_Oral_subsample_50_894	{'C113_2'}



Oral_51: C159_1
CAMI_Oral_subsample_50_735
CAMI_Oral_subsample_50_818
CAMI_Oral_subsample_50_734
CAMI_Oral_subsample_10_308
CAMI_Oral_subsample_50_279

'''



import os

def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


###################################################### file in/out #####################################################

wd                  = '/Users/songweizhi/Desktop/MarkerMAG_wd/4_Kelp'
taxonomy_mag        = '%s/BH_ER_050417.bac120.summary.tsv'                              % wd
taxonomy_16s        = '%s/Kelp_SILVA128_id95_assembled_16S_uclust_0.999_classified.txt' % wd
taxonomy_16s_blca   = '%s/Kelp_SILVA128_id95_assembled_16S_uclust_0.999.fasta.blca.out' % wd
bin_id_file         = '%s/bin_id.txt'                                                   % wd
barrnap_output      = '%s/BH_ER_050417_16S.txt'                                         % wd
checkm_output       = '%s/BH_ER_050417_refined_MAG_qualities.txt'                       % wd
mean_depth_file     = '%s/BH_ER_050417_refined_bins_mean_depth.txt'                     % wd

linkage_file        = '%s/file_in/Kelp_70_3_128_combined_linkages.txt'                  % wd


'''
Linkage accuracy: Kelp_NewCigar_70_mis1_combined_linkages
Rank	Links	NA	Correct	Unknown	Total(Accuracy)
phylum	37	0	37	0	37(100.0)
class	37	0	34	0	34(91.89)
order	37	0	30	0	30(81.08)
family	37	0	28	0	28(75.68)
genus	37	0	18	7	25(67.57)

Linkage accuracy: Kelp_NewCigar_70_mis2_combined_linkages
Rank	Links	NA	Correct	Unknown	Total(Accuracy)
phylum	46	0	46	0	46(100.0)
class	46	0	43	0	43(93.48)
order	46	0	39	0	39(84.78)
family	46	0	37	0	37(80.43)
genus	46	0	27	7	34(73.91)

Linkage accuracy: Kelp_NewCigar_70_mis3_combined_linkages
Rank	Links	NA	Correct	Unknown	Total(Accuracy)
phylum	50	0	50	0	50(100.0)
class	50	0	47	0	47(94.0)
order	50	0	43	0	43(86.0)
family	50	0	41	0	41(82.0)
genus	50	0	31	7	38(76.0)

        70_mis1         70_mis2         70_mis3         70_mis4         70_mis5
phylum	37/37(100.0)	46/46(100.0)	50/50(100.0)	52/52(100.0)    54/54(100.0)
class	34/37(91.89)	43/46(93.48)	47/50(94.0)	    49/52(94.23)    51/54(94.44)
order	30/37(81.08)	39/46(84.78)	43/50(86.0) 	45/52(86.54)    47/54(87.04)
family	28/37(75.68)	37/46(80.43)	41/50(82.0) 	43/52(82.69)    45/54(83.33)
genus	18/37(48.65)	27/46(58.7)	    31/50(62.0) 	32/52(61.54)    34/54(62.96)

80_mis1
phylum	33/33(100.0)
class	31/33(93.94)
order	27/33(81.82)
family	25/33(75.76)
genus	17/33(51.52)

80_mis2
phylum	42/42(100.0)
class	40/42(95.24)
order	36/42(85.71)
family	34/42(80.95)
genus	24/42(57.14)


80_mis3
phylum	44/44(100.0)
class	42/44(95.45)
order	38/44(86.36)
family	36/44(81.82)
genus	26/44(59.09)

60_mis1
phylum	38/38(100.0)
class	35/38(92.11)
order	31/38(81.58)
family	29/38(76.32)
genus	18/38(47.37)

60_mis2
phylum	48/48(100.0)
class	45/48(93.75)
order	41/48(85.42)
family	39/48(81.25)
genus	29/48(60.42)

60_mis3
phylum	46/46(100.0)
class	43/46(93.48)
order	39/46(84.78)
family	37/46(80.43)
genus	27/46(58.7)

'''

################################################### define file name ###################################################

linkage_file_path, linkage_file_basename, linkage_file_extension = sep_path_basename_ext(linkage_file)
linkage_file_with_assessment        = '%s/%s_with_assessment%s'         % (linkage_file_path, linkage_file_basename, linkage_file_extension)
linkage_file_with_assessment_by_mag = '%s/%s_with_assessment_by_MAG%s'  % (linkage_file_path, linkage_file_basename, linkage_file_extension)


################################################### read in metadata ###################################################


bin_id_list = []
for each_bin_id in open(bin_id_file):
    bin_id_list.append(each_bin_id.strip())

bin_taxon_dict = {}
for each_bin_taxon in open(taxonomy_mag):
    if not each_bin_taxon.startswith('user_genome	classification'):
        each_bin_taxon_split = each_bin_taxon.strip().split('\t')
        bin_taxon_dict[each_bin_taxon_split[0]] = each_bin_taxon_split[1]

mag_completeness_dict = {}
for each_mag_quality in open(checkm_output):
    each_mag_quality_split = each_mag_quality.strip().split(',')
    mag_completeness_dict[each_mag_quality_split[0]] = each_mag_quality_split[1]

bin_depth_dict = {}
for each_bin_depth in open(mean_depth_file):
    each_bin_depth_split = each_bin_depth.strip().split('\t')
    bin_depth_dict[each_bin_depth_split[0]] = each_bin_depth_split[2]

barrnap_op_dict = {}
for each_barrnap_op in open(barrnap_output):
    if not each_barrnap_op.startswith('Genome	16S	Length'):
        each_barrnap_op_split = each_barrnap_op.strip().split('\t')
        barrnap_op_dict[each_barrnap_op_split[0]] = each_barrnap_op_split[2]

s16_taxon_dict = {}
for each_16s_taxon in open(taxonomy_16s):
    each_16s_taxon_split = each_16s_taxon.strip().split('\t')
    s16_taxon_dict[each_16s_taxon_split[0]] = each_16s_taxon_split[1].split(' [')[0]

s16_taxon_blca_dict = {}
for each_16s_taxon in open(taxonomy_16s_blca):
    each_16s_taxon_split = each_16s_taxon.strip().split('\t')
    s16_taxon_blca_dict[each_16s_taxon_split[0]] = each_16s_taxon_split[1]

s16_taxon_blca_dict_formatted = {}
for each_16s in s16_taxon_blca_dict:
    taxon_blca_raw = s16_taxon_blca_dict[each_16s]
    formatted_taxon_str = 'Unclassified'
    if taxon_blca_raw != 'Unclassified':
        taxon_blca_raw_split_1 = taxon_blca_raw.strip().split(':')[1:]
        formatted_taxon_list = []
        for each_str in taxon_blca_raw_split_1:
            each_str_split = each_str.split(';')
            taxon_with_confidence = '%s(%s)' % (each_str_split[0], each_str_split[1][:5])
            formatted_taxon_list.append(taxon_with_confidence)
        formatted_taxon_str = ';'.join(formatted_taxon_list)
    s16_taxon_blca_dict_formatted[each_16s] = formatted_taxon_str


###################################################### by linkage ######################################################

linkage_file_with_assessment_handle = open(linkage_file_with_assessment, 'w')
total_linkage = 0
correct_linkage_p = 0
linkage_with_unknown_p = 0
correct_linkage_c = 0
linkage_with_unknown_c = 0
correct_linkage_o = 0
linkage_with_unknown_o = 0
correct_linkage_f = 0
linkage_with_unknown_f = 0
correct_linkage_g = 0
linkage_with_unknown_g = 0
correct_linkage_s = 0
linkage_with_unknown_s = 0
unknown_mag_or_16s_taxon = 0
for each_linkage in open(linkage_file):
    if each_linkage.startswith('MarkerGene	GenomicSeq'):
        linkage_file_with_assessment_handle.write('%s\tPhylum\tClass\tOrder\tFamily\tGenus\tMAG_taxon\t16S_taxon\t16S_taxon_BLCA\n' % each_linkage.strip())
    else:
        each_linkage_split = each_linkage.strip().split('\t')
        line_to_write = each_linkage.strip()
        s16_id = each_linkage_split[0]
        mag_id = each_linkage_split[1]
        taxon_mag = bin_taxon_dict.get(mag_id, 'NA')
        taxon_16s = s16_taxon_dict.get(s16_id, 'NA')
        taxon_16s_blca = s16_taxon_blca_dict_formatted.get(s16_id, 'NA')
        taxon_mag_split = taxon_mag.split(';')
        taxon_16s_split = taxon_16s.split(';')
        total_linkage += 1

        if (taxon_mag == 'NA') or (taxon_16s == 'NA'):
            unknown_mag_or_16s_taxon += 1
        else:
            taxon_mag_p = taxon_mag_split[1]
            taxon_mag_c = taxon_mag_split[2]
            taxon_mag_o = taxon_mag_split[3]
            taxon_mag_f = taxon_mag_split[4]
            taxon_mag_g = taxon_mag_split[5]
            taxon_mag_s = taxon_mag_split[6]
            taxon_16s_p = taxon_16s_split[1]
            taxon_16s_c = taxon_16s_split[2]
            taxon_16s_o = taxon_16s_split[3]
            taxon_16s_f = taxon_16s_split[4]
            taxon_16s_g = taxon_16s_split[5]
            taxon_16s_s = taxon_16s_split[6]
            already_wrong = False

            if taxon_mag_p == taxon_16s_p:
                correct_linkage_p += 1
                line_to_write += '\t1'
            else:
                if taxon_mag_p == 'p__':
                    linkage_with_unknown_p += 1
                    line_to_write += '\tna'
                else:
                    already_wrong = True
                    line_to_write += '\t0'

            if already_wrong is False:
                if taxon_mag_c == taxon_16s_c:
                    correct_linkage_c += 1
                    line_to_write += '\t1'
                else:
                    if taxon_mag_c == 'c__':
                        linkage_with_unknown_c += 1
                        line_to_write += '\tna'
                    else:
                        already_wrong = True
                        line_to_write += '\t0'
            else:
                line_to_write += '\t0'

            if already_wrong is False:
                if taxon_mag_o == taxon_16s_o:
                    correct_linkage_o += 1
                    line_to_write += '\t1'
                else:
                    if taxon_mag_o == 'o__':
                        linkage_with_unknown_o += 1
                        line_to_write += '\tna'
                    else:
                        already_wrong = True
                        line_to_write += '\t0'
            else:
                line_to_write += '\t0'

            if already_wrong is False:
                if taxon_mag_f == taxon_16s_f:
                    correct_linkage_f += 1
                    line_to_write += '\t1'
                else:
                    if taxon_mag_f == 'f__':
                        linkage_with_unknown_f += 1
                        line_to_write += '\tna'
                    else:
                        already_wrong = True
                        line_to_write += '\t0'
            else:
                line_to_write += '\t0'

            if already_wrong is False:
                if taxon_mag_g == taxon_16s_g:
                    correct_linkage_g += 1
                    line_to_write += '\t1'
                else:
                    if taxon_mag_g == 'g__':
                        linkage_with_unknown_g += 1
                        line_to_write += '\tna'
                    else:
                        already_wrong = True
                        line_to_write += '\t0'
            else:
                line_to_write += '\t0'

        line_to_write += '\t%s\t%s\t%s' % (taxon_mag, taxon_16s, taxon_16s_blca)
        linkage_file_with_assessment_handle.write('%s\n' % line_to_write)


if total_linkage > 0:

    print('=============================')
    accuracy_p = float("{0:.2f}".format((correct_linkage_p + linkage_with_unknown_p) * 100 / (total_linkage - unknown_mag_or_16s_taxon)))
    accuracy_c = float("{0:.2f}".format((correct_linkage_c + linkage_with_unknown_c) * 100 / (total_linkage - unknown_mag_or_16s_taxon)))
    accuracy_o = float("{0:.2f}".format((correct_linkage_o + linkage_with_unknown_o) * 100 / (total_linkage - unknown_mag_or_16s_taxon)))
    accuracy_f = float("{0:.2f}".format((correct_linkage_f + linkage_with_unknown_f) * 100 / (total_linkage - unknown_mag_or_16s_taxon)))
    accuracy_g = float("{0:.2f}".format((correct_linkage_g + linkage_with_unknown_g) * 100 / (total_linkage - unknown_mag_or_16s_taxon)))

    print('Linkage accuracy: %s' % linkage_file_basename)
    print('Rank\tLinks\tNA\tCorrect\tUnknown\tTotal(Accuracy)')
    print('phylum\t%s\t%s\t%s\t%s\t%s(%s)' % (total_linkage, unknown_mag_or_16s_taxon, correct_linkage_p, linkage_with_unknown_p, (correct_linkage_p + linkage_with_unknown_p), accuracy_p))
    print('class\t%s\t%s\t%s\t%s\t%s(%s)'  % (total_linkage, unknown_mag_or_16s_taxon, correct_linkage_c, linkage_with_unknown_c, (correct_linkage_c + linkage_with_unknown_c), accuracy_c))
    print('order\t%s\t%s\t%s\t%s\t%s(%s)'  % (total_linkage, unknown_mag_or_16s_taxon, correct_linkage_o, linkage_with_unknown_o, (correct_linkage_o + linkage_with_unknown_o), accuracy_o))
    print('family\t%s\t%s\t%s\t%s\t%s(%s)' % (total_linkage, unknown_mag_or_16s_taxon, correct_linkage_f, linkage_with_unknown_f, (correct_linkage_f + linkage_with_unknown_f), accuracy_f))
    print('genus\t%s\t%s\t%s\t%s\t%s(%s)'  % (total_linkage, unknown_mag_or_16s_taxon, correct_linkage_g, linkage_with_unknown_g, (correct_linkage_g + linkage_with_unknown_g), accuracy_g))
    print('=============================')

######################################################## by MAG ########################################################

linked_mag_list = set()
for each_linkage in open(linkage_file):
    if not each_linkage.startswith('MarkerGene	GenomicSeq	Linkage	Step'):
        each_linkage_split = each_linkage.strip().split('\t')
        mag_id = each_linkage_split[1]
        linked_mag_list.add(mag_id)

linkage_file_with_assessment_by_mag_handle = open(linkage_file_with_assessment_by_mag, 'w')
linkage_file_with_assessment_by_mag_handle.write('MAG\tdepth\t16S\tcompleteness\tlinked\ttaxon\n')
for each_mag in sorted(bin_id_list):

    mag_depth = '0'
    if each_mag in bin_depth_dict:
        mag_depth = bin_depth_dict[each_mag]

    mag_taxon = 'unknown'
    if each_mag in bin_taxon_dict:
        mag_taxon = bin_taxon_dict[each_mag]

    with_16s_in_mag = 'no'
    if each_mag in barrnap_op_dict:
        with_16s_in_mag = barrnap_op_dict[each_mag]

    linked_to_16s = 'no'
    if each_mag in linked_mag_list:
        linked_to_16s = 'yes'

    mag_cpl = 0
    if each_mag in mag_completeness_dict:
        mag_cpl = mag_completeness_dict[each_mag]

    linkage_file_with_assessment_by_mag_handle.write('%s\t%s\t%s\t%s\t%s\t%s\n' % (each_mag, mag_depth, with_16s_in_mag, mag_cpl, linked_to_16s, mag_taxon))
linkage_file_with_assessment_by_mag_handle.close()

print('linked MAGs: %s' % len(linked_mag_list))



import os

bin_id_file = '/Users/songweizhi/Desktop/refined_MAGs_prefix.txt'

for each in open(bin_id_file):
    bin_id = each.strip()
    add_prefix_cmd = 'blastn -query /srv/scratch/z5039045/MarkerMAG_wd/CAMI_high/CAMI_high_refined_bins_renamed/%s.fasta -db combined_refs.fna -out %s_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 12' % (bin_id, bin_id)
    print(add_prefix_cmd)


# /srv/scratch/z5039045/MarkerMAG_wd/CAMI_high/blast_between_bin_and_ref/combined_refs.fna



def get_GapFilling_stats_by_assembly(free_living_16s_ref_file,
                                     free_living_ctg_ref_file,
                                     mini_assembly_to_16s_reads,
                                     mini_assembly_to_ctg_reads,
                                     ctg_level_min_link,
                                     mini_assembly_to_16s_ctg_connector,
                                     gnm_to_ctg_connector,
                                     marker_to_ctg_gnm_Key_connector,
                                     stats_GapFilling_ctg,
                                     stats_GapFilling_gnm):

    max_within_cate_diff_pct = 80
    max_between_cate_diff_pct = 20

    round2_free_living_16s_ref_dict = {}
    for free_living_read_16s in open(free_living_16s_ref_file):
        free_living_read_16s_split = free_living_read_16s.strip().split('\t')
        if len(free_living_read_16s_split) > 1:
            read_16s_id = free_living_read_16s_split[0]
            read_16s_refs = free_living_read_16s_split[1].split(',')
            round2_free_living_16s_ref_dict[read_16s_id] = read_16s_refs

    round2_free_living_ctg_ref_dict = {}
    for free_living_read_ctg in open(free_living_ctg_ref_file):
        free_living_read_ctg_split = free_living_read_ctg.strip().split('\t')
        read_ctg_id = free_living_read_ctg_split[0]
        read_ctg_refs = free_living_read_ctg_split[1].split(',')
        read_ctg_refs_no_suffix = []
        for each_read_ctg_ref in read_ctg_refs:
            if each_read_ctg_ref[-2:] in ['_l', '_r']:
                each_read_ctg_ref_no_suffix = each_read_ctg_ref[:-2]
                read_ctg_refs_no_suffix.append(each_read_ctg_ref_no_suffix)
        round2_free_living_ctg_ref_dict[read_ctg_id] = read_ctg_refs_no_suffix

    mini_assembly_to_16s_dict = {}
    for each_mini_assembly in open(mini_assembly_to_16s_reads):
        mini_assembly_split = each_mini_assembly.strip().split('\t')
        mini_assembly_id = mini_assembly_split[0]
        mini_assembly_mapped_reads = mini_assembly_split[1].split(',')
        for each_mapped_read in mini_assembly_mapped_reads:
            mapped_read_16s_refs = round2_free_living_16s_ref_dict.get(each_mapped_read, [])
            for each_mapped_read_16s_ref in mapped_read_16s_refs:
                mini_assembly_to_16s_key = '%s%s%s' % (mini_assembly_id, mini_assembly_to_16s_ctg_connector, each_mapped_read_16s_ref)
                if mini_assembly_to_16s_key not in mini_assembly_to_16s_dict:
                    mini_assembly_to_16s_dict[mini_assembly_to_16s_key] = 1
                else:
                    mini_assembly_to_16s_dict[mini_assembly_to_16s_key] += 1

    mini_assembly_to_ctg_dict = {}
    for each_mini_assembly in open(mini_assembly_to_ctg_reads):
        mini_assembly_split = each_mini_assembly.strip().split('\t')
        mini_assembly_id = mini_assembly_split[0]
        mini_assembly_mapped_reads = mini_assembly_split[1].split(',')
        for each_mapped_read in mini_assembly_mapped_reads:
            mapped_read_ctg_refs = round2_free_living_ctg_ref_dict.get(each_mapped_read, [])
            for each_mapped_read_ctg_ref in mapped_read_ctg_refs:
                mini_assembly_to_ctg_key = '%s%s%s' % (mini_assembly_id, mini_assembly_to_16s_ctg_connector, each_mapped_read_ctg_ref)
                if mini_assembly_to_ctg_key not in mini_assembly_to_ctg_dict:
                    mini_assembly_to_ctg_dict[mini_assembly_to_ctg_key] = 1
                else:
                    mini_assembly_to_ctg_dict[mini_assembly_to_ctg_key] += 1

    mini_assembly_to_16s_dict_reformatted = {}
    max_link_nun_dict_16s = {}
    for each in mini_assembly_to_16s_dict:
        mini_assembly_id = each.split(mini_assembly_to_16s_ctg_connector)[0]
        seq_16s_id = each.split(mini_assembly_to_16s_ctg_connector)[1]
        linkage_num = mini_assembly_to_16s_dict[each]
        seq_16s_with_num = '%s__num__%s' % (seq_16s_id, linkage_num)
        if linkage_num >= ctg_level_min_link:

            # add to mini_assembly_to_16s_dict_reformatted
            if mini_assembly_id not in mini_assembly_to_16s_dict_reformatted:
                mini_assembly_to_16s_dict_reformatted[mini_assembly_id] = {seq_16s_with_num}
            else:
                mini_assembly_to_16s_dict_reformatted[mini_assembly_id].add(seq_16s_with_num)

            # add to max_link_nun_dict_16s
            if seq_16s_id not in max_link_nun_dict_16s:
                max_link_nun_dict_16s[seq_16s_id] = linkage_num
            else:
                if linkage_num > max_link_nun_dict_16s[seq_16s_id]:
                    max_link_nun_dict_16s[seq_16s_id] = linkage_num

    mini_assembly_to_ctg_dict_reformatted = {}
    max_link_nun_dict_ctg = {}
    for each in mini_assembly_to_ctg_dict:
        mini_assembly_id = each.split(mini_assembly_to_16s_ctg_connector)[0]
        ctg_id = each.split(mini_assembly_to_16s_ctg_connector)[1]
        linkage_num = mini_assembly_to_ctg_dict[each]
        ctg_with_num = '%s__num__%s' % (ctg_id, linkage_num)
        if linkage_num >= ctg_level_min_link:

            # add to mini_assembly_to_ctg_dict_reformatted
            if mini_assembly_id not in mini_assembly_to_ctg_dict_reformatted:
                mini_assembly_to_ctg_dict_reformatted[mini_assembly_id] = {ctg_with_num}
            else:
                mini_assembly_to_ctg_dict_reformatted[mini_assembly_id].add(ctg_with_num)

            # add to max_link_nun_dict_ctg
            if ctg_id not in max_link_nun_dict_ctg:
                max_link_nun_dict_ctg[ctg_id] = linkage_num
            else:
                if linkage_num > max_link_nun_dict_ctg[ctg_id]:
                    max_link_nun_dict_ctg[ctg_id] = linkage_num

    mini_assembly_linked_both = set(mini_assembly_to_16s_dict_reformatted).intersection(mini_assembly_to_ctg_dict_reformatted)


    ######################### debug report #########################

    check_list_gnm  = []
    check_list_16s  = []

    check_list_gnm = ['Refined_51', 'Refined_91']
    check_list_16s = ['3_GI_subsample_50_882', '3_GI_subsample_75_963']

    report_all      = False

    '''
    # 3_GI_subsample_75_963
    NODE_4066_length_298_cov_0.076023	3_GI_subsample_75_963(20)	Refined_28___C___NODE_4309_length_14678_cov_0.076146(6)
    NODE_660_length_449_cov_0.232919	3_GI_subsample_75_963(8)	Refined_84___C___NODE_14371_length_3638_cov_0.021646(3)
    NODE_660_length_449_cov_0.232919	3_GI_subsample_75_963(8)	Refined_91___C___NODE_3602_length_17682_cov_0.132270(6)
    NODE_12_length_1555_cov_0.052521	3_GI_subsample_75_963(160)	Refined_51___C___NODE_7061_length_8073_cov_0.110244(51)
    NODE_12_length_1555_cov_0.052521	3_GI_subsample_75_963(160)	Refined_28___C___NODE_4309_length_14678_cov_0.076146(96)
    
    # Refined_28
 
    '''

    ################################################################

    # for report
    error_report_linked_gnm_list = []
    error_report_linked_16s_list = []
    stats_GapFilling_ctg_handle = open(stats_GapFilling_ctg, 'w')
    stats_GapFilling_gnm_dict = {}
    for each_mini_assembly in mini_assembly_linked_both:
        linked_16s = mini_assembly_to_16s_dict_reformatted[each_mini_assembly]
        linked_ctg = mini_assembly_to_ctg_dict_reformatted[each_mini_assembly]
        linked_16s_num_list = [int(i.split('__num__')[1]) for i in linked_16s]
        linked_ctg_num_list = [int(i.split('__num__')[1]) for i in linked_ctg]
        linked_16s_num_max = max(linked_16s_num_list)
        linked_ctg_num_max = max(linked_ctg_num_list)


        for each_linked_16s in linked_16s:
            linked_16s_id = each_linked_16s.split('__num__')[0]
            linked_16s_num = int(each_linked_16s.split('__num__')[1])
            for each_linked_ctg in linked_ctg:
                linked_ctg_id = each_linked_ctg.split('__num__')[0]
                linked_gnm_id = linked_ctg_id.split(gnm_to_ctg_connector)[0]
                linked_ctg_num = int(each_linked_ctg.split('__num__')[1])
                error_report_str_linked_gnm = '%s\t%s(%s)\t%s(%s)' % (each_mini_assembly, linked_16s_id, linked_16s_num, linked_gnm_id, linked_ctg_num)
                error_report_str_linked_16s = '%s\t%s(%s)\t%s(%s)' % (each_mini_assembly, linked_16s_id, linked_16s_num, linked_ctg_id, linked_ctg_num)

                if report_all is True:
                    if linked_gnm_id in check_list_gnm:
                        error_report_linked_gnm_list.append(error_report_str_linked_gnm)
                    if linked_16s_id in check_list_16s:
                        error_report_linked_16s_list.append(error_report_str_linked_16s)


        if (min(linked_16s_num_max, linked_ctg_num_max) * 100 / max(linked_16s_num_max, linked_ctg_num_max)) >= max_between_cate_diff_pct:

            linked_16s_filtered = [i for i in linked_16s if int(i.split('__num__')[1])*100/linked_16s_num_max >= max_within_cate_diff_pct]
            linked_ctg_filtered = [i for i in linked_ctg if int(i.split('__num__')[1])*100/linked_ctg_num_max >= max_within_cate_diff_pct]

            for each_linked_16s in linked_16s_filtered:
                linked_16s_id = each_linked_16s.split('__num__')[0]
                linked_16s_num = int(each_linked_16s.split('__num__')[1])
                for each_linked_ctg in linked_ctg_filtered:
                    linked_ctg_id = each_linked_ctg.split('__num__')[0]
                    linked_gnm_id = linked_ctg_id.split(gnm_to_ctg_connector)[0]
                    linked_ctg_num = int(each_linked_ctg.split('__num__')[1])

                    linked_16s_max_num = max_link_nun_dict_16s[linked_16s_id]
                    linked_ctg_max_num = max_link_nun_dict_ctg[linked_ctg_id]
                    error_report_str_linked_gnm = '%s\t%s(%s)\t%s(%s)' % (each_mini_assembly, linked_16s_id, linked_16s_num, linked_gnm_id, linked_ctg_num)
                    error_report_str_linked_16s = '%s\t%s(%s)\t%s(%s)' % (each_mini_assembly, linked_16s_id, linked_16s_num, linked_ctg_id, linked_ctg_num)

                    if report_all is False:
                        if linked_gnm_id in check_list_gnm:
                            error_report_linked_gnm_list.append(error_report_str_linked_gnm)
                        if linked_16s_id in check_list_16s:
                            error_report_linked_16s_list.append(error_report_str_linked_16s)

                    linked_16s_num_pct_by_max = linked_16s_num*100/linked_16s_max_num
                    linked_ctg_num_pct_by_max = linked_ctg_num*100/linked_ctg_max_num

                    if (linked_16s_num_pct_by_max >= 50) and (linked_ctg_num_pct_by_max >= 50):
                        stats_GapFilling_ctg_handle.write('%s\t%s\t%s\n' % (linked_16s_id, linked_ctg_id, (linked_16s_num + linked_ctg_num)))
                        marker_to_gnm_key = '%s%s%s' % (linked_16s_id, marker_to_ctg_gnm_Key_connector, linked_gnm_id)
                        if marker_to_gnm_key not in stats_GapFilling_gnm_dict:
                            stats_GapFilling_gnm_dict[marker_to_gnm_key] = (linked_16s_num + linked_ctg_num)
                        else:
                            stats_GapFilling_gnm_dict[marker_to_gnm_key] += (linked_16s_num + linked_ctg_num)
    stats_GapFilling_ctg_handle.close()

    ######################### debug report #########################


    if check_list_gnm != []:
        print('Mini-assembly linked to:\t%s' % ', '.join(check_list_gnm))
        for each_link_gnm in error_report_linked_gnm_list:
            print(each_link_gnm)
        print()

    if check_list_16s != []:
        print('Mini-assembly linked to:\t%s' % ', '.join(check_list_16s))
        for each_link_16s in error_report_linked_16s_list:
            print(each_link_16s)
        print()

    ################################################################

    stats_GapFilling_gnm_handle = open(stats_GapFilling_gnm, 'w')
    stats_GapFilling_gnm_handle.write('MarkerGene,GenomicSeq,Number\n')
    for each_16s_to_gnm in stats_GapFilling_gnm_dict:
        each_16s_to_gnm_split = each_16s_to_gnm.split(marker_to_ctg_gnm_Key_connector)
        id_16s = each_16s_to_gnm_split[0]
        id_gnm = each_16s_to_gnm_split[1]
        linkage_num = stats_GapFilling_gnm_dict[each_16s_to_gnm]
        stats_GapFilling_gnm_handle.write('MarkerGene__%s,GenomicSeq__%s,%s\n' % (id_16s, id_gnm, linkage_num))
    stats_GapFilling_gnm_handle.close()

'''
14
Mini-assembly linked to:	Refined_28
NODE_419_length_586_cov_0.065359	3_GI_subsample_25_431(14)	Refined_28(45)
NODE_4801_length_285_cov_0.082278	3_GI_subsample_50_581(26)	Refined_28(6)
NODE_4801_length_285_cov_0.082278	3_GI_subsample_75_961(29)	Refined_28(6)
NODE_2373_length_336_cov_0.062201	3_GI_subsample_50_782(11)	Refined_28(5)
NODE_2373_length_336_cov_0.062201	3_GI_subsample_100_1646(13)	Refined_28(5)

15
Mini-assembly linked to:	Refined_28
NODE_4066_length_298_cov_0.076023	3_GI_subsample_75_961(29)	Refined_28(6)
NODE_4066_length_298_cov_0.076023	3_GI_subsample_50_581(26)	Refined_28(6)
NODE_12_length_1555_cov_0.052521	3_GI_subsample_75_963(160)	Refined_28(96)
NODE_2978_length_321_cov_0.000000	3_GI_subsample_100_1646(13)	Refined_28(5)
NODE_2978_length_321_cov_0.000000	3_GI_subsample_50_782(11)	Refined_28(5)
NODE_417_length_586_cov_0.089325	3_GI_subsample_25_431(14)	Refined_28(45)

'''


step_2_wd                   = '/Users/songweizhi/Desktop/51_91'
free_living_16s_ref_file    = '%s/round2_free_living_16s_refs.txt'      % step_2_wd
free_living_ctg_ref_file    = '%s/round2_free_living_ctg_refs.txt'      % step_2_wd
mini_assembly_to_16s_reads  = '%s/mini_assembly_to_16s_reads.txt'       % step_2_wd
mini_assembly_to_ctg_reads  = '%s/mini_assembly_to_ctg_reads.txt'       % step_2_wd
stats_GapFilling_ctg        = '%s/stats_GapFilling_ctg.txt'             % step_2_wd
stats_GapFilling_gnm        = '%s/stats_GapFilling_gnm.txt'             % step_2_wd
ctg_level_min_link          = 3

mini_assembly_to_16s_ctg_connector  = '___Mini___'
gnm_to_ctg_connector                = '___C___'
marker_to_ctg_gnm_Key_connector     = '___M___'


get_GapFilling_stats_by_assembly(free_living_16s_ref_file,
                                 free_living_ctg_ref_file,
                                 mini_assembly_to_16s_reads,
                                 mini_assembly_to_ctg_reads,
                                 ctg_level_min_link,
                                 mini_assembly_to_16s_ctg_connector,
                                 gnm_to_ctg_connector,
                                 marker_to_ctg_gnm_Key_connector,
                                 stats_GapFilling_ctg,
                                 stats_GapFilling_gnm)


'''

3_GI_subsample_75_963(169)  --- NODE_2265_length_348_cov_0.257919   ---     Refined_51___C___NODE_7061_length_8073_cov_0.110244(44)
3_GI_subsample_75_963(125)  --- NODE_126_length_934_cov_0.154895    ---     Refined_51___C___NODE_7061_length_8073_cov_0.110244(48)

3_GI_subsample_50_882(9)	--- NODE_3017_length_328_cov_0.144279   ---     Refined_91___C___NODE_3602_length_17682_cov_0.132270(33)



'''









'''
contig id:  NODE_11_length_1555_cov_0.052521
cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/GI_0418_mis2_no_clp_MarkerMAG_wd/GI_0418_mis2_no_clp_step_2_wd
BioSAK select_seq -seq mini_assembly_SPAdes_wd/scaffolds.fasta -id NODE_11_length_1555_cov_0.052521.txt -out NODE_11_length_1555_cov_0.052521.fa -option 1
BioSAK select_seq -seq /srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/3_GI_Matam16S_wd/3_GI_assembled_16S_uclust_0.999.fasta -id 3_GI_subsample_75_963.txt -out 3_GI_subsample_75_963.fa -option 1
BioSAK select_seq -seq /srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/GI_refined_bins/Refined_28.fasta -id NODE_4309_length_14678_cov_0.076146.txt -out NODE_4309_length_14678_cov_0.076146.fa -option 1


cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/GI_0418_mis2_no_clp_MarkerMAG_wd/GI_0418_mis2_no_clp_step_2_wd
blastn -query 3_GI_subsample_75_963.fa -subject /srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/ref_genomes_renamed.fa -out 3_GI_subsample_75_963_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn
blastn -query NODE_11_length_1555_cov_0.052521.fa -subject /srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/ref_genomes_renamed.fa -out NODE_11_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn
blastn -query NODE_4309_length_14678_cov_0.076146.fa -subject /srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/ref_genomes_renamed.fa -out NODE_4309_vs_ref.tab -evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn


# ctg matched to (not mis-binned ctg)
CP002770.1  OTU_97.3402.0
Refined_28	OTU_97.3402.0
NODE_4309_length_14678_cov_0.076146	    CP002770.1	    99.986	14678	2	0	1	14678	701936	687259	0.0	26462	14678	3601386

# 16S matched to 
3_GI_subsample_75_963	FR745875.1	100.000	1494	0	0	1	1494	3691677	3690184	0.0	2695	1494	3781509
3_GI_subsample_75_963	CP001056.1	100.000	1494	0	0	1	1494	3691664	3690171	0.0	2695	1494	3800327
3_GI_subsample_75_963	CP001078.1	99.933	1494	1	0	1	1494	3543149	3541656	0.0	2691	1494	3659644
3_GI_subsample_75_963	CP010521.1	99.933	1494	1	0	1	1494	3495403	3493910	0.0	2691	1494	3611898
3_GI_subsample_75_963	CP010520.1	99.933	1494	1	0	1	1494	3495402	3493909	0.0	2691	1494	3611897
3_GI_subsample_75_963	CP006903.1	99.866	1494	2	0	1	1494	2392237	2390744	0.0	2686	1494	3874462

# mini-assembly matched to
NODE_11_length_1555_cov_0.052521	FR745875.1	99.486	1555	8	0	1	1555	3704931	3703377	0.0	2769	1555	3781509
NODE_11_length_1555_cov_0.052521	CP001056.1	99.550	1555	7	0	1	1555	9725	11279	0.0	2773	1555	3800327
NODE_11_length_1555_cov_0.052521	CP001078.1	99.550	1555	7	0	1	1555	3424658	3423104	0.0	2773	1555	3659644
NODE_11_length_1555_cov_0.052521	CP010521.1	99.550	1555	7	0	1	1555	3377284	3375730	0.0	2773	1555	3611898
NODE_11_length_1555_cov_0.052521	CP010520.1	99.550	1555	7	0	1	1555	3377283	3375729	0.0	2773	1555	3611897
NODE_11_length_1555_cov_0.052521	CP006903.1	99.357	1555	9	1	1	1555	2392131	2390578	0.0	2756	1555	3874462










'''





from Bio import SeqIO

fa_file = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/CAMI2_GI_mplu5_longkmer_55-127_2nd_MarkerMAG_wd/CAMI2_GI_mplu5_longkmer_55-127_2nd_step_2_wd/free_living_read_combined.fasta'
fq_r1 = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/1_filtered_reads/GI_R1.fastq'
fq_r2 = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/1_filtered_reads/GI_R2.fastq'
fq_out = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI2_HMP/CAMI2_GI_mplu5_longkmer_55-127_2nd_MarkerMAG_wd/CAMI2_GI_mplu5_longkmer_55-127_2nd_step_2_wd/free_living_read_combined.fastq'


reads_to_extract = set()
for fa_read_id in SeqIO.parse(fa_file, 'fasta'):
    reads_to_extract.add(fa_read_id.id)

fq_out_handle = open(fq_out, 'w')
for r1_record in SeqIO.parse(fq_r1, 'fastq'):
    if r1_record.id in reads_to_extract:
        SeqIO.write(r1_record, fq_out_handle, 'fastq')
for r2_record in SeqIO.parse(fq_r2, 'fastq'):
    if r2_record.id in reads_to_extract:
        SeqIO.write(r2_record, fq_out_handle, 'fastq')
fq_out_handle.close()







def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if each_element.isalpha() is True:
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


pwd_samfile = '/Users/songweizhi/Desktop/MBARC26_all_depth_assemblies.dereplicated_99.5_500bp.sam'
clipping_reads_not_matched_part_seq = '/Users/songweizhi/Desktop/not_matched_part.fasta'
min_cigar_M = 30
min_cigar_S = 30


# export clipping mapped reads and perfectly mapped reads
clipping_mapped_reads_list = set()
clipping_reads_mapped_part_dict = {}
perfectly_mapped_reads_dict = {}
clipping_reads_not_matched_part_seq_handle = open(clipping_reads_not_matched_part_seq, 'w')
for each_read in open(pwd_samfile):

    if not each_read.startswith('@'):
        each_read_split = each_read.strip().split('\t')
        read_id = each_read_split[0]
        read_id_base = '.'.join(read_id.split('.')[:-1])
        read_strand = read_id.split('.')[-1]
        ref_id = each_read_split[2]
        ref_id_with_prefix = 'MarkerGene__%s' % each_read_split[2]
        ref_pos = int(each_read_split[3])
        cigar = each_read_split[5]
        read_seq = each_read_split[9]
        cigar_splitted = cigar_splitter(cigar)
        read_id_with_ref_pos = '%s__x__%s__x__%s' % (read_id, ref_id, ref_pos)

        # for perfectly mapped reads
        if ('M' in cigar) and (len(cigar_splitted) == 1):
            if read_id_base not in perfectly_mapped_reads_dict:
                perfectly_mapped_reads_dict[read_id_base] = {read_strand: [ref_id_with_prefix]}
            else:
                if read_strand not in perfectly_mapped_reads_dict[read_id_base]:
                    perfectly_mapped_reads_dict[read_id_base][read_strand] = [ref_id_with_prefix]
                else:
                    perfectly_mapped_reads_dict[read_id_base][read_strand].append(ref_id_with_prefix)

        # for clipping mapped reads
        if ('S' in cigar) and (len(cigar_splitted) == 2):

            # get the length of mapped and unmapped parts
            cigar_M_len = 0
            cigar_S_len = 0
            split_pos = 0
            if cigar_splitted[0][-1] == 'M':
                cigar_M_len = int(cigar_splitted[0][:-1])
                cigar_S_len = int(cigar_splitted[1][:-1])
                split_pos = ref_pos + cigar_M_len
            if cigar_splitted[1][-1] == 'M':
                cigar_M_len = int(cigar_splitted[1][:-1])
                cigar_S_len = int(cigar_splitted[0][:-1])
                split_pos = ref_pos

            # for clipping reads with unmapped part >= min_cigar_S
            if (cigar_M_len >= min_cigar_M) and (cigar_S_len >= min_cigar_S):
                read_seq_left = read_seq[: int(cigar_splitted[0][:-1])]
                read_seq_right = read_seq[-int(cigar_splitted[1][:-1]):]

                if cigar_splitted[0][-1] == 'M':

                    # write out the sequence of unmapped part
                    clipping_reads_not_matched_part_seq_handle.write('>%s_r\n' % read_id_with_ref_pos)
                    clipping_reads_not_matched_part_seq_handle.write(read_seq_right + '\n')

                    # store the match info of mapped part
                    if ('%s_l' % read_id_with_ref_pos) not in clipping_reads_mapped_part_dict:
                        clipping_reads_mapped_part_dict[('%s_l' % read_id_with_ref_pos)] = [ref_id_with_prefix]
                    else:
                        clipping_reads_mapped_part_dict[('%s_l' % read_id_with_ref_pos)].append(ref_id_with_prefix)

                if cigar_splitted[1][-1] == 'M':

                    # write out the sequence of unmapped part
                    clipping_reads_not_matched_part_seq_handle.write('>%s_l\n' % read_id_with_ref_pos)
                    clipping_reads_not_matched_part_seq_handle.write(read_seq_left + '\n')

                    # store the match info of mapped part
                    if ('%s_r' % read_id_with_ref_pos) not in clipping_reads_mapped_part_dict:
                        clipping_reads_mapped_part_dict[('%s_r' % read_id_with_ref_pos)] = [ref_id_with_prefix]
                    else:
                        clipping_reads_mapped_part_dict[('%s_r' % read_id_with_ref_pos)].append(ref_id_with_prefix)

                clipping_mapped_reads_list.add(read_id)

            # for clipping reads with unmapped part < min_cigar_S
            # treat these reads as perfectly mapped reads
            if (cigar_M_len >= min_cigar_M) and (cigar_S_len < min_cigar_S):

                if read_id_base not in perfectly_mapped_reads_dict:
                    perfectly_mapped_reads_dict[read_id_base] = {read_strand: [ref_id_with_prefix]}
                else:
                    if read_strand not in perfectly_mapped_reads_dict[read_id_base]:
                        perfectly_mapped_reads_dict[read_id_base][read_strand] = [ref_id_with_prefix]
                    else:
                        perfectly_mapped_reads_dict[read_id_base][read_strand].append(ref_id_with_prefix)





clipping_reads_not_matched_part_seq_handle.close()

print(perfectly_mapped_reads_dict)


print(len(perfectly_mapped_reads_dict))


import argparse
from Bio import SeqIO

# BINID in gs_read_mapping_1.binning refers to cami_genome_id

parser = argparse.ArgumentParser()
parser.add_argument('-i', required=True, type=int, help='sample_index')
args = vars(parser.parse_args())
sample_index = args['i']

wd = '.'
qualified_gnm_id_file   = '%s/ref_with_16S_1000bp_5000bp_416.txt'   % wd
gs_read_mapping_file    = '%s/gs_read_mapping_%s.binning'           % (wd, sample_index)
r1_in                   = '%s/RH_S00%s_R1.fa'                       % (wd, sample_index)
r2_in                   = '%s/RH_S00%s_R2.fa'                       % (wd, sample_index)
r1_out                  = '%s/RH_S00%s_R1_filtered.fa'              % (wd, sample_index)
r2_out                  = '%s/RH_S00%s_R2_filtered.fa'              % (wd, sample_index)

qualified_gnm_id_set = set()
for each_ref in open(qualified_gnm_id_file):
    qualified_gnm_id_set.add(each_ref.strip())

read_to_source_dict = {}
for each_read in open(gs_read_mapping_file):
    if (not each_read.startswith('@')) and (len(each_read) > 1):
        each_read_split = each_read.strip().split('\t')
        read_base = each_read_split[0]
        ref_id = each_read_split[1]
        read_to_source_dict[read_base] = ref_id

r1_out_handle = open(r1_out, 'w')
for each_read in SeqIO.parse(r1_in, 'fasta'):
    read_id = each_read.id
    read_id_base = read_id.split('/')[0]
    read_source = read_to_source_dict.get(read_id_base, 'na')
    if read_source in qualified_gnm_id_set:
        r1_out_handle.write('>%s\n' % read_id)
        r1_out_handle.write('%s\n' % str(each_read.seq))
r1_out_handle.close()

r2_out_handle = open(r2_out, 'w')
for each_read in SeqIO.parse(r2_in, 'fasta'):
    read_id = each_read.id
    read_id_base = read_id.split('/')[0]
    read_source = read_to_source_dict.get(read_id_base, 'na')
    if read_source in qualified_gnm_id_set:
        r2_out_handle.write('>%s\n' % read_id)
        r2_out_handle.write('%s\n' % str(each_read.seq))
r2_out_handle.close()
